@@ -26,7 +26,7 @@ use arrow::datatypes::SchemaRef;
 use arrow::json::{ArrayWriter, LineDelimitedWriter};
 use arrow::record_batch::RecordBatch;
 use arrow::util::pretty::pretty_format_batches_with_options;
-use datafusion::common::format::DEFAULT_FORMAT_OPTIONS;
+use datafusion::common::format::DEFAULT_CLI_FORMAT_OPTIONS;
 use datafusion::error::Result;
 
 /// Allow records to be printed in different formats
@@ -133,7 +133,7 @@ fn format_batches_with_maxrows<W: std::io::Write>(
 
             let formatted = pretty_format_batches_with_options(
                 &filtered_batches,
-                &DEFAULT_FORMAT_OPTIONS,
+                &DEFAULT_CLI_FORMAT_OPTIONS,
             )?;
             if over_limit {
                 let mut formatted_str = format!("{}", formatted);
@@ -145,7 +145,7 @@ fn format_batches_with_maxrows<W: std::io::Write>(
         }
         MaxRows::Unlimited => {
             let formatted =
-                pretty_format_batches_with_options(batches, &DEFAULT_FORMAT_OPTIONS)?;
+                pretty_format_batches_with_options(batches, &DEFAULT_CLI_FORMAT_OPTIONS)?;
             writeln!(writer, "{}", formatted)?;
         }
     }
@@ -201,7 +201,7 @@ impl PrintFormat {
                 let empty_batch = RecordBatch::new_empty(schema);
                 let formatted = pretty_format_batches_with_options(
                     &[empty_batch],
-                    &DEFAULT_FORMAT_OPTIONS,
+                    &DEFAULT_CLI_FORMAT_OPTIONS,
                 )?;
                 writeln!(writer, "{}", formatted)?;
             }
