@@ -27,7 +27,8 @@ use datafusion::common::{
     internal_datafusion_err, DFSchema, DataFusionError, Result, ScalarValue,
 };
 use datafusion::datasource::listing::PartitionedFile;
-use datafusion::datasource::physical_plan::{FileScanConfig, ParquetSource};
+use datafusion::datasource::memory::DataSourceExec;
+use datafusion::datasource::physical_plan::{FileScanConfigBuilder, ParquetSource};
 use datafusion::datasource::TableProvider;
 use datafusion::execution::object_store::ObjectStoreUrl;
 use datafusion::logical_expr::{
@@ -243,8 +244,8 @@ impl TableProvider for IndexTableProvider {
         let object_store_url = ObjectStoreUrl::parse("file://")?;
         let source =
             Arc::new(ParquetSource::default().with_predicate(self.schema(), predicate));
-        let mut file_scan_config =
-            FileScanConfig::new(object_store_url, self.schema(), source)
+        let mut file_scan_config_builder =
+            FileScanConfigBuilder::new(object_store_url, self.schema(), source)
                 .with_projection(projection.cloned())
                 .with_limit(limit);
 
@@ -253,12 +254,13 @@ impl TableProvider for IndexTableProvider {
         for (file_name, file_size) in files {
             let path = self.dir.join(file_name);
             let canonical_path = fs::canonicalize(path)?;
-            file_scan_config = file_scan_config.with_file(PartitionedFile::new(
-                canonical_path.display().to_string(),
-                file_size,
-            ));
+            file_scan_config_builder = file_scan_config_builder.with_file(
+                PartitionedFile::new(canonical_path.display().to_string(), file_size),
+            );
         }
-        Ok(file_scan_config.build())
+        Ok(DataSourceExec::from_data_source(
+            file_scan_config_builder.build(),
+        ))
     }
 
     /// Tell DataFusion to push filters down to the scan method
@@ -683,7 +685,7 @@ fn make_demo_file(path: impl AsRef<Path>, value_range: Range<i32>) -> Result<()>
 
     let num_values = value_range.len();
     let file_names =
-        StringArray::from_iter_values(std::iter::repeat(&filename).take(num_values));
+        StringArray::from_iter_values(std::iter::repeat_n(&filename, num_values));
     let values = Int32Array::from_iter_values(value_range);
     let batch = RecordBatch::try_from_iter(vec![
         ("file_name", Arc::new(file_names) as ArrayRef),
