@@ -21,17 +21,17 @@ use async_trait::async_trait;
 use datafusion::arrow::datatypes::SchemaRef;
 use datafusion::arrow::record_batch::RecordBatch;
 use datafusion::catalog::Session;
+use datafusion::catalog::TableFunctionImpl;
+use datafusion::common::{plan_err, ScalarValue};
+use datafusion::datasource::memory::MemorySourceConfig;
 use datafusion::datasource::TableProvider;
 use datafusion::error::Result;
 use datafusion::execution::context::ExecutionProps;
-use datafusion::physical_plan::memory::MemoryExec;
+use datafusion::logical_expr::simplify::SimplifyContext;
+use datafusion::logical_expr::{Expr, TableType};
+use datafusion::optimizer::simplify_expressions::ExprSimplifier;
 use datafusion::physical_plan::ExecutionPlan;
-use datafusion::prelude::SessionContext;
-use datafusion_catalog::TableFunctionImpl;
-use datafusion_common::{plan_err, ScalarValue};
-use datafusion_expr::simplify::SimplifyContext;
-use datafusion_expr::{Expr, TableType};
-use datafusion_optimizer::simplify_expressions::ExprSimplifier;
+use datafusion::prelude::*;
 use std::fs::File;
 use std::io::Seek;
 use std::path::Path;
@@ -120,11 +120,11 @@ impl TableProvider for LocalCsvTable {
         } else {
             self.batches.clone()
         };
-        Ok(Arc::new(MemoryExec::try_new(
+        Ok(MemorySourceConfig::try_new_exec(
             &[batches],
             TableProvider::schema(self),
             projection.cloned(),
-        )?))
+        )?)
     }
 }
 
