@@ -29,8 +29,7 @@ use parking_lot::Mutex;
 use std::sync::Arc;
 use tokio::runtime::Runtime;
 
-fn query(ctx: Arc<Mutex<SessionContext>>, sql: &str) {
-    let rt = Runtime::new().unwrap();
+fn query(ctx: Arc<Mutex<SessionContext>>, rt: &Runtime, sql: &str) {
     let df = rt.block_on(ctx.lock().sql(sql)).unwrap();
     criterion::black_box(rt.block_on(df.collect()).unwrap());
 }
@@ -51,11 +50,13 @@ fn criterion_benchmark(c: &mut Criterion) {
     let array_len = 32768 * 2; // 2^16
     let batch_size = 2048; // 2^11
     let ctx = create_context(partitions_len, array_len, batch_size).unwrap();
+    let rt = Runtime::new().unwrap();
 
     c.bench_function("aggregate_query_no_group_by 15 12", |b| {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT MIN(f64), AVG(f64), COUNT(f64) \
                  FROM t",
             )
@@ -66,6 +67,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT MIN(f64), MAX(f64) \
                  FROM t",
             )
@@ -76,6 +78,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT COUNT(DISTINCT u64_wide) \
                  FROM t",
             )
@@ -86,6 +89,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT COUNT(DISTINCT u64_narrow) \
                  FROM t",
             )
@@ -96,6 +100,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT utf8, MIN(f64), AVG(f64), COUNT(f64) \
                  FROM t GROUP BY utf8",
             )
@@ -106,6 +111,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT utf8, MIN(f64), AVG(f64), COUNT(f64) \
                  FROM t \
                  WHERE f32 > 10 AND f32 < 20 GROUP BY utf8",
@@ -117,6 +123,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT u64_narrow, MIN(f64), AVG(f64), COUNT(f64) \
                  FROM t GROUP BY u64_narrow",
             )
@@ -127,6 +134,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT u64_narrow, MIN(f64), AVG(f64), COUNT(f64) \
                  FROM t \
                  WHERE f32 > 10 AND f32 < 20 GROUP BY u64_narrow",
@@ -138,6 +146,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT u64_wide, utf8, MIN(f64), AVG(f64), COUNT(f64) \
                  FROM t GROUP BY u64_wide, utf8",
             )
@@ -148,6 +157,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT utf8, approx_percentile_cont(u64_wide, 0.5, 2500)  \
                  FROM t GROUP BY utf8",
             )
@@ -158,6 +168,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT utf8, approx_percentile_cont(f32, 0.5, 2500)  \
                  FROM t GROUP BY utf8",
             )
@@ -168,6 +179,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT MEDIAN(DISTINCT u64_wide), MEDIAN(DISTINCT u64_narrow) \
                  FROM t",
             )
@@ -178,6 +190,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT first_value(u64_wide order by f64, u64_narrow, utf8),\
                             last_value(u64_wide order by f64, u64_narrow, utf8)  \
                  FROM t GROUP BY u64_narrow",
@@ -189,6 +202,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT first_value(u64_wide ignore nulls order by f64, u64_narrow, utf8),  \
                             last_value(u64_wide ignore nulls order by f64, u64_narrow, utf8)    \
                  FROM t GROUP BY u64_narrow",
@@ -200,6 +214,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         b.iter(|| {
             query(
                 ctx.clone(),
+                &rt,
                 "SELECT first_value(u64_wide order by f64), \
                             last_value(u64_wide order by f64)   \
                 FROM t GROUP BY u64_narrow",
