@@ -15,453 +15,46 @@
 // specific language governing permissions and limitations
 // under the License.
 
-//! Execution plan for reading line-delimited JSON files
+//! Reexports the [`datafusion_datasource_json::source`] module, containing JSON based [`FileSource`].
+//!
+//! [`FileSource`]: datafusion_datasource::file::FileSource
 
-use std::any::Any;
-use std::io::{BufReader, Read, Seek, SeekFrom};
-use std::sync::Arc;
-use std::task::Poll;
-
-use super::{calculate_range, RangeCalculation};
-
-use crate::datasource::file_format::file_compression_type::FileCompressionType;
-use crate::datasource::file_format::{deserialize_stream, DecoderDeserializer};
-use crate::datasource::listing::{ListingTableUrl, PartitionedFile};
-use crate::datasource::physical_plan::FileMeta;
-use crate::datasource::physical_plan::{FileOpenFuture, FileOpener};
-use crate::error::{DataFusionError, Result};
-use crate::physical_plan::{ExecutionPlan, ExecutionPlanProperties};
-
-use arrow::json::ReaderBuilder;
-use arrow::{datatypes::SchemaRef, json};
-use datafusion_common::{Constraints, Statistics};
-use datafusion_datasource::file::FileSource;
-use datafusion_datasource::file_scan_config::FileScanConfig;
-use datafusion_datasource::source::DataSourceExec;
-use datafusion_execution::{SendableRecordBatchStream, TaskContext};
-use datafusion_physical_expr::{EquivalenceProperties, Partitioning};
-use datafusion_physical_expr_common::sort_expr::LexOrdering;
-use datafusion_physical_plan::execution_plan::{Boundedness, EmissionType};
-use datafusion_physical_plan::metrics::{ExecutionPlanMetricsSet, MetricsSet};
-use datafusion_physical_plan::{DisplayAs, DisplayFormatType, PlanProperties};
-
-use futures::{StreamExt, TryStreamExt};
-use object_store::buffered::BufWriter;
-use object_store::{GetOptions, GetResultPayload, ObjectStore};
-use tokio::io::AsyncWriteExt;
-use tokio::task::JoinSet;
-
-/// Execution plan for scanning NdJson data source
-#[derive(Debug, Clone)]
-#[deprecated(since = "46.0.0", note = "use DataSourceExec instead")]
-pub struct NdJsonExec {
-    inner: DataSourceExec,
-    base_config: FileScanConfig,
-    file_compression_type: FileCompressionType,
-}
-
-#[allow(unused, deprecated)]
-impl NdJsonExec {
-    /// Create a new JSON reader execution plan provided base configurations
-    pub fn new(
-        base_config: FileScanConfig,
-        file_compression_type: FileCompressionType,
-    ) -> Self {
-        let (
-            projected_schema,
-            projected_constraints,
-            projected_statistics,
-            projected_output_ordering,
-        ) = base_config.project();
-        let cache = Self::compute_properties(
-            projected_schema,
-            &projected_output_ordering,
-            projected_constraints,
-            &base_config,
-        );
-
-        let json = JsonSource::default();
-        let base_config = base_config
-            .with_file_compression_type(file_compression_type)
-            .with_source(Arc::new(json));
-
-        Self {
-            inner: DataSourceExec::new(Arc::new(base_config.clone())),
-            file_compression_type: base_config.file_compression_type,
-            base_config,
-        }
-    }
-
-    /// Ref to the base configs
-    pub fn base_config(&self) -> &FileScanConfig {
-        &self.base_config
-    }
-
-    /// Ref to file compression type
-    pub fn file_compression_type(&self) -> &FileCompressionType {
-        &self.file_compression_type
-    }
-
-    fn file_scan_config(&self) -> FileScanConfig {
-        self.inner
-            .data_source()
-            .as_any()
-            .downcast_ref::<FileScanConfig>()
-            .unwrap()
-            .clone()
-    }
-
-    fn json_source(&self) -> JsonSource {
-        let source = self.file_scan_config();
-        source
-            .file_source()
-            .as_any()
-            .downcast_ref::<JsonSource>()
-            .unwrap()
-            .clone()
-    }
-
-    fn output_partitioning_helper(file_scan_config: &FileScanConfig) -> Partitioning {
-        Partitioning::UnknownPartitioning(file_scan_config.file_groups.len())
-    }
-
-    /// This function creates the cache object that stores the plan properties such as schema, equivalence properties, ordering, partitioning, etc.
-    fn compute_properties(
-        schema: SchemaRef,
-        orderings: &[LexOrdering],
-        constraints: Constraints,
-        file_scan_config: &FileScanConfig,
-    ) -> PlanProperties {
-        // Equivalence Properties
-        let eq_properties = EquivalenceProperties::new_with_orderings(schema, orderings)
-            .with_constraints(constraints);
-
-        PlanProperties::new(
-            eq_properties,
-            Self::output_partitioning_helper(file_scan_config), // Output Partitioning
-            EmissionType::Incremental,
-            Boundedness::Bounded,
-        )
-    }
-
-    fn with_file_groups(mut self, file_groups: Vec<Vec<PartitionedFile>>) -> Self {
-        self.base_config.file_groups = file_groups.clone();
-        let mut file_source = self.file_scan_config();
-        file_source = file_source.with_file_groups(file_groups);
-        self.inner = self.inner.with_data_source(Arc::new(file_source));
-        self
-    }
-}
-
-#[allow(unused, deprecated)]
-impl DisplayAs for NdJsonExec {
-    fn fmt_as(
-        &self,
-        t: DisplayFormatType,
-        f: &mut std::fmt::Formatter,
-    ) -> std::fmt::Result {
-        self.inner.fmt_as(t, f)
-    }
-}
-
-#[allow(unused, deprecated)]
-impl ExecutionPlan for NdJsonExec {
-    fn name(&self) -> &'static str {
-        "NdJsonExec"
-    }
-
-    fn as_any(&self) -> &dyn Any {
-        self
-    }
-    fn properties(&self) -> &PlanProperties {
-        self.inner.properties()
-    }
-
-    fn children(&self) -> Vec<&Arc<dyn ExecutionPlan>> {
-        Vec::new()
-    }
-
-    fn with_new_children(
-        self: Arc<Self>,
-        _: Vec<Arc<dyn ExecutionPlan>>,
-    ) -> Result<Arc<dyn ExecutionPlan>> {
-        Ok(self)
-    }
-
-    fn repartitioned(
-        &self,
-        target_partitions: usize,
-        config: &datafusion_common::config::ConfigOptions,
-    ) -> Result<Option<Arc<dyn ExecutionPlan>>> {
-        self.inner.repartitioned(target_partitions, config)
-    }
-
-    fn execute(
-        &self,
-        partition: usize,
-        context: Arc<TaskContext>,
-    ) -> Result<SendableRecordBatchStream> {
-        self.inner.execute(partition, context)
-    }
-
-    fn statistics(&self) -> Result<Statistics> {
-        self.inner.statistics()
-    }
-
-    fn metrics(&self) -> Option<MetricsSet> {
-        self.inner.metrics()
-    }
-
-    fn fetch(&self) -> Option<usize> {
-        self.inner.fetch()
-    }
-
-    fn with_fetch(&self, limit: Option<usize>) -> Option<Arc<dyn ExecutionPlan>> {
-        self.inner.with_fetch(limit)
-    }
-}
-
-/// A [`FileOpener`] that opens a JSON file and yields a [`FileOpenFuture`]
-pub struct JsonOpener {
-    batch_size: usize,
-    projected_schema: SchemaRef,
-    file_compression_type: FileCompressionType,
-    object_store: Arc<dyn ObjectStore>,
-}
-
-impl JsonOpener {
-    /// Returns a  [`JsonOpener`]
-    pub fn new(
-        batch_size: usize,
-        projected_schema: SchemaRef,
-        file_compression_type: FileCompressionType,
-        object_store: Arc<dyn ObjectStore>,
-    ) -> Self {
-        Self {
-            batch_size,
-            projected_schema,
-            file_compression_type,
-            object_store,
-        }
-    }
-}
-
-/// JsonSource holds the extra configuration that is necessary for [`JsonOpener`]
-#[derive(Clone, Default)]
-pub struct JsonSource {
-    batch_size: Option<usize>,
-    metrics: ExecutionPlanMetricsSet,
-    projected_statistics: Option<Statistics>,
-}
-
-impl JsonSource {
-    /// Initialize a JsonSource with default values
-    pub fn new() -> Self {
-        Self::default()
-    }
-}
-
-impl FileSource for JsonSource {
-    fn create_file_opener(
-        &self,
-        object_store: Arc<dyn ObjectStore>,
-        base_config: &FileScanConfig,
-        _partition: usize,
-    ) -> Arc<dyn FileOpener> {
-        Arc::new(JsonOpener {
-            batch_size: self
-                .batch_size
-                .expect("Batch size must set before creating opener"),
-            projected_schema: base_config.projected_file_schema(),
-            file_compression_type: base_config.file_compression_type,
-            object_store,
-        })
-    }
-
-    fn as_any(&self) -> &dyn Any {
-        self
-    }
-
-    fn with_batch_size(&self, batch_size: usize) -> Arc<dyn FileSource> {
-        let mut conf = self.clone();
-        conf.batch_size = Some(batch_size);
-        Arc::new(conf)
-    }
-
-    fn with_schema(&self, _schema: SchemaRef) -> Arc<dyn FileSource> {
-        Arc::new(Self { ..self.clone() })
-    }
-    fn with_statistics(&self, statistics: Statistics) -> Arc<dyn FileSource> {
-        let mut conf = self.clone();
-        conf.projected_statistics = Some(statistics);
-        Arc::new(conf)
-    }
-
-    fn with_projection(&self, _config: &FileScanConfig) -> Arc<dyn FileSource> {
-        Arc::new(Self { ..self.clone() })
-    }
-
-    fn metrics(&self) -> &ExecutionPlanMetricsSet {
-        &self.metrics
-    }
-
-    fn statistics(&self) -> Result<Statistics> {
-        let statistics = &self.projected_statistics;
-        Ok(statistics
-            .clone()
-            .expect("projected_statistics must be set to call"))
-    }
-
-    fn file_type(&self) -> &str {
-        "json"
-    }
-}
-
-impl FileOpener for JsonOpener {
-    /// Open a partitioned NDJSON file.
-    ///
-    /// If `file_meta.range` is `None`, the entire file is opened.
-    /// Else `file_meta.range` is `Some(FileRange{start, end})`, which corresponds to the byte range [start, end) within the file.
-    ///
-    /// Note: `start` or `end` might be in the middle of some lines. In such cases, the following rules
-    /// are applied to determine which lines to read:
-    /// 1. The first line of the partition is the line in which the index of the first character >= `start`.
-    /// 2. The last line of the partition is the line in which the byte at position `end - 1` resides.
-    ///
-    /// See [`CsvOpener`](super::CsvOpener) for an example.
-    fn open(&self, file_meta: FileMeta) -> Result<FileOpenFuture> {
-        let store = Arc::clone(&self.object_store);
-        let schema = Arc::clone(&self.projected_schema);
-        let batch_size = self.batch_size;
-        let file_compression_type = self.file_compression_type.to_owned();
-
-        Ok(Box::pin(async move {
-            let calculated_range = calculate_range(&file_meta, &store, None).await?;
-
-            let range = match calculated_range {
-                RangeCalculation::Range(None) => None,
-                RangeCalculation::Range(Some(range)) => Some(range.into()),
-                RangeCalculation::TerminateEarly => {
-                    return Ok(
-                        futures::stream::poll_fn(move |_| Poll::Ready(None)).boxed()
-                    )
-                }
-            };
-
-            let options = GetOptions {
-                range,
-                ..Default::default()
-            };
-
-            let result = store.get_opts(file_meta.location(), options).await?;
-
-            match result.payload {
-                GetResultPayload::File(mut file, _) => {
-                    let bytes = match file_meta.range {
-                        None => file_compression_type.convert_read(file)?,
-                        Some(_) => {
-                            file.seek(SeekFrom::Start(result.range.start as _))?;
-                            let limit = result.range.end - result.range.start;
-                            file_compression_type.convert_read(file.take(limit as u64))?
-                        }
-                    };
-
-                    let reader = ReaderBuilder::new(schema)
-                        .with_batch_size(batch_size)
-                        .build(BufReader::new(bytes))?;
-
-                    Ok(futures::stream::iter(reader).boxed())
-                }
-                GetResultPayload::Stream(s) => {
-                    let s = s.map_err(DataFusionError::from);
-
-                    let decoder = ReaderBuilder::new(schema)
-                        .with_batch_size(batch_size)
-                        .build_decoder()?;
-                    let input = file_compression_type.convert_stream(s.boxed())?.fuse();
-
-                    Ok(deserialize_stream(
-                        input,
-                        DecoderDeserializer::from(decoder),
-                    ))
-                }
-            }
-        }))
-    }
-}
-
-pub async fn plan_to_json(
-    task_ctx: Arc<TaskContext>,
-    plan: Arc<dyn ExecutionPlan>,
-    path: impl AsRef<str>,
-) -> Result<()> {
-    let path = path.as_ref();
-    let parsed = ListingTableUrl::parse(path)?;
-    let object_store_url = parsed.object_store();
-    let store = task_ctx.runtime_env().object_store(&object_store_url)?;
-    let mut join_set = JoinSet::new();
-    for i in 0..plan.output_partitioning().partition_count() {
-        let storeref = Arc::clone(&store);
-        let plan: Arc<dyn ExecutionPlan> = Arc::clone(&plan);
-        let filename = format!("{}/part-{i}.json", parsed.prefix());
-        let file = object_store::path::Path::parse(filename)?;
-
-        let mut stream = plan.execute(i, Arc::clone(&task_ctx))?;
-        join_set.spawn(async move {
-            let mut buf_writer = BufWriter::new(storeref, file.clone());
-
-            let mut buffer = Vec::with_capacity(1024);
-            while let Some(batch) = stream.next().await.transpose()? {
-                let mut writer = json::LineDelimitedWriter::new(buffer);
-                writer.write(&batch)?;
-                buffer = writer.into_inner();
-                buf_writer.write_all(&buffer).await?;
-                buffer.clear();
-            }
-
-            buf_writer.shutdown().await.map_err(DataFusionError::from)
-        });
-    }
-
-    while let Some(result) = join_set.join_next().await {
-        match result {
-            Ok(res) => res?, // propagate DataFusion error
-            Err(e) => {
-                if e.is_panic() {
-                    std::panic::resume_unwind(e.into_panic());
-                } else {
-                    unreachable!();
-                }
-            }
-        }
-    }
-
-    Ok(())
-}
+#[allow(deprecated)]
+pub use datafusion_datasource_json::source::*;
 
 #[cfg(test)]
 mod tests {
-    use std::fs;
-    use std::path::Path;
 
     use super::*;
+
+    use std::fs;
+    use std::path::Path;
+    use std::sync::Arc;
+
     use crate::dataframe::DataFrameWriteOptions;
-    use crate::datasource::file_format::{json::JsonFormat, FileFormat};
-    use crate::datasource::listing::PartitionedFile;
-    use crate::datasource::object_store::ObjectStoreUrl;
-    use crate::execution::context::SessionState;
-    use crate::prelude::{
-        CsvReadOptions, NdJsonReadOptions, SessionConfig, SessionContext,
-    };
+    use crate::execution::SessionState;
+    use crate::prelude::{CsvReadOptions, NdJsonReadOptions, SessionContext};
     use crate::test::partitioned_file_groups;
-    use crate::{assert_batches_eq, assert_batches_sorted_eq};
+    use datafusion_common::cast::{as_int32_array, as_int64_array, as_string_array};
+    use datafusion_common::test_util::batches_to_string;
+    use datafusion_common::Result;
+    use datafusion_datasource::file_compression_type::FileCompressionType;
+    use datafusion_datasource::file_format::FileFormat;
+    use datafusion_datasource_json::JsonFormat;
+    use datafusion_execution::config::SessionConfig;
+    use datafusion_execution::object_store::ObjectStoreUrl;
+    use datafusion_physical_plan::ExecutionPlan;
 
     use arrow::array::Array;
+    use arrow::datatypes::SchemaRef;
     use arrow::datatypes::{Field, SchemaBuilder};
-    use datafusion_common::cast::{as_int32_array, as_int64_array, as_string_array};
-
+    use datafusion_datasource::file_groups::FileGroup;
+    use datafusion_datasource::file_scan_config::FileScanConfigBuilder;
+    use datafusion_datasource::source::DataSourceExec;
+    use insta::assert_snapshot;
     use object_store::chunked::ChunkedStore;
     use object_store::local::LocalFileSystem;
+    use object_store::ObjectStore;
     use rstest::*;
     use tempfile::TempDir;
     use url::Url;
@@ -472,7 +65,7 @@ mod tests {
         state: &SessionState,
         file_compression_type: FileCompressionType,
         work_dir: &Path,
-    ) -> (ObjectStoreUrl, Vec<Vec<PartitionedFile>>, SchemaRef) {
+    ) -> (ObjectStoreUrl, Vec<FileGroup>, SchemaRef) {
         let store_url = ObjectStoreUrl::local_filesystem();
         let store = state.runtime_env().object_store(&store_url).unwrap();
 
@@ -489,6 +82,7 @@ mod tests {
         let meta = file_groups
             .first()
             .unwrap()
+            .files()
             .first()
             .unwrap()
             .clone()
@@ -523,6 +117,7 @@ mod tests {
         let path = file_groups
             .first()
             .unwrap()
+            .files()
             .first()
             .unwrap()
             .object_meta
@@ -544,19 +139,17 @@ mod tests {
         let frame = ctx.read_json(path, read_options).await.unwrap();
         let results = frame.collect().await.unwrap();
 
-        assert_batches_eq!(
-            &[
-                "+-----+------------------+---------------+------+",
-                "| a   | b                | c             | d    |",
-                "+-----+------------------+---------------+------+",
-                "| 1   | [2.0, 1.3, -6.1] | [false, true] | 4    |",
-                "| -10 | [2.0, 1.3, -6.1] | [true, true]  | 4    |",
-                "| 2   | [2.0, , -6.1]    | [false, ]     | text |",
-                "|     |                  |               |      |",
-                "+-----+------------------+---------------+------+",
-            ],
-            &results
-        );
+        insta::allow_duplicates! {assert_snapshot!(batches_to_string(&results), @r###"
+            +-----+------------------+---------------+------+
+            | a   | b                | c             | d    |
+            +-----+------------------+---------------+------+
+            | 1   | [2.0, 1.3, -6.1] | [false, true] | 4    |
+            | -10 | [2.0, 1.3, -6.1] | [true, true]  | 4    |
+            | 2   | [2.0, , -6.1]    | [false, ]     | text |
+            |     |                  |               |      |
+            +-----+------------------+---------------+------+
+        "###);}
+
         Ok(())
     }
 
@@ -578,16 +171,19 @@ mod tests {
         let task_ctx = session_ctx.task_ctx();
         use arrow::datatypes::DataType;
 
+        use futures::StreamExt;
+
         let tmp_dir = TempDir::new()?;
         let (object_store_url, file_groups, file_schema) =
             prepare_store(&state, file_compression_type.to_owned(), tmp_dir.path()).await;
 
         let source = Arc::new(JsonSource::new());
-        let conf = FileScanConfig::new(object_store_url, file_schema, source)
+        let conf = FileScanConfigBuilder::new(object_store_url, file_schema, source)
             .with_file_groups(file_groups)
             .with_limit(Some(3))
-            .with_file_compression_type(file_compression_type.to_owned());
-        let exec = conf.build();
+            .with_file_compression_type(file_compression_type.to_owned())
+            .build();
+        let exec = DataSourceExec::from_data_source(conf);
 
         // TODO: this is not where schema inference should be tested
 
@@ -638,10 +234,13 @@ mod tests {
     async fn nd_json_exec_file_with_missing_column(
         file_compression_type: FileCompressionType,
     ) -> Result<()> {
+        use arrow::datatypes::DataType;
+
+        use futures::StreamExt;
+
         let session_ctx = SessionContext::new();
         let state = session_ctx.state();
         let task_ctx = session_ctx.task_ctx();
-        use arrow::datatypes::DataType;
 
         let tmp_dir = TempDir::new()?;
         let (object_store_url, file_groups, actual_schema) =
@@ -654,11 +253,12 @@ mod tests {
         let missing_field_idx = file_schema.fields.len() - 1;
 
         let source = Arc::new(JsonSource::new());
-        let conf = FileScanConfig::new(object_store_url, file_schema, source)
+        let conf = FileScanConfigBuilder::new(object_store_url, file_schema, source)
             .with_file_groups(file_groups)
             .with_limit(Some(3))
-            .with_file_compression_type(file_compression_type.to_owned());
-        let exec = conf.build();
+            .with_file_compression_type(file_compression_type.to_owned())
+            .build();
+        let exec = DataSourceExec::from_data_source(conf);
 
         let mut it = exec.execute(0, task_ctx)?;
         let batch = it.next().await.unwrap()?;
@@ -686,6 +286,8 @@ mod tests {
     async fn nd_json_exec_file_projection(
         file_compression_type: FileCompressionType,
     ) -> Result<()> {
+        use futures::StreamExt;
+
         let session_ctx = SessionContext::new();
         let state = session_ctx.state();
         let task_ctx = session_ctx.task_ctx();
@@ -694,11 +296,12 @@ mod tests {
             prepare_store(&state, file_compression_type.to_owned(), tmp_dir.path()).await;
 
         let source = Arc::new(JsonSource::new());
-        let conf = FileScanConfig::new(object_store_url, file_schema, source)
+        let conf = FileScanConfigBuilder::new(object_store_url, file_schema, source)
             .with_file_groups(file_groups)
             .with_projection(Some(vec![0, 2]))
-            .with_file_compression_type(file_compression_type.to_owned());
-        let exec = conf.build();
+            .with_file_compression_type(file_compression_type.to_owned())
+            .build();
+        let exec = DataSourceExec::from_data_source(conf);
         let inferred_schema = exec.schema();
         assert_eq!(inferred_schema.fields().len(), 2);
 
@@ -731,6 +334,8 @@ mod tests {
     async fn nd_json_exec_file_mixed_order_projection(
         file_compression_type: FileCompressionType,
     ) -> Result<()> {
+        use futures::StreamExt;
+
         let session_ctx = SessionContext::new();
         let state = session_ctx.state();
         let task_ctx = session_ctx.task_ctx();
@@ -739,11 +344,12 @@ mod tests {
             prepare_store(&state, file_compression_type.to_owned(), tmp_dir.path()).await;
 
         let source = Arc::new(JsonSource::new());
-        let conf = FileScanConfig::new(object_store_url, file_schema, source)
+        let conf = FileScanConfigBuilder::new(object_store_url, file_schema, source)
             .with_file_groups(file_groups)
             .with_projection(Some(vec![3, 0, 2]))
-            .with_file_compression_type(file_compression_type.to_owned());
-        let exec = conf.build();
+            .with_file_compression_type(file_compression_type.to_owned())
+            .build();
+        let exec = DataSourceExec::from_data_source(conf);
         let inferred_schema = exec.schema();
         assert_eq!(inferred_schema.fields().len(), 3);
 
@@ -889,7 +495,7 @@ mod tests {
             .write_json(out_dir_url, DataFrameWriteOptions::new(), None)
             .await
             .expect_err("should fail because input file does not match inferred schema");
-        assert_eq!(e.strip_backtrace(), "Arrow error: Parser error: Error while parsing value d for column 0 at line 4");
+        assert_eq!(e.strip_backtrace(), "Arrow error: Parser error: Error while parsing value 'd' as type 'Int64' for column 0 at line 4. Row data: '[d,4]'");
         Ok(())
     }
 
@@ -936,6 +542,9 @@ mod tests {
     async fn test_json_with_repartitioning(
         file_compression_type: FileCompressionType,
     ) -> Result<()> {
+        use datafusion_common::assert_batches_sorted_eq;
+        use datafusion_execution::config::SessionConfig;
+
         let config = SessionConfig::new()
             .with_repartition_file_scans(true)
             .with_repartition_file_min_size(0)
@@ -957,6 +566,7 @@ mod tests {
         let path = file_groups
             .first()
             .unwrap()
+            .files()
             .first()
             .unwrap()
             .object_meta
