@@ -17,18 +17,18 @@
 
 //! [`SessionContext`] API for registering data sources and executing queries
 
-use datafusion_catalog::memory::MemorySchemaProvider;
-use datafusion_catalog::MemoryCatalogProvider;
 use std::collections::HashSet;
 use std::fmt::Debug;
 use std::sync::{Arc, Weak};
 
 use super::options::ReadOptions;
+use crate::datasource::dynamic_file::DynamicListTableFactory;
+use crate::execution::session_state::SessionStateBuilder;
 use crate::{
+    catalog::listing_schema::ListingSchemaProvider,
     catalog::{
         CatalogProvider, CatalogProviderList, TableProvider, TableProviderFactory,
     },
-    catalog_common::listing_schema::ListingSchemaProvider,
     dataframe::DataFrame,
     datasource::listing::{
         ListingOptions, ListingTable, ListingTableConfig, ListingTableUrl,
@@ -49,46 +49,52 @@ use crate::{
     variable::{VarProvider, VarType},
 };
 
+// backwards compatibility
+pub use crate::execution::session_state::SessionState;
+
 use arrow::datatypes::{Schema, SchemaRef};
 use arrow::record_batch::RecordBatch;
+use datafusion_catalog::memory::MemorySchemaProvider;
+use datafusion_catalog::MemoryCatalogProvider;
+use datafusion_catalog::{
+    DynamicFileCatalog, TableFunction, TableFunctionImpl, UrlTableFactory,
+};
+use datafusion_common::config::ConfigOptions;
 use datafusion_common::{
     config::{ConfigExtension, TableOptions},
     exec_datafusion_err, exec_err, not_impl_err, plan_datafusion_err, plan_err,
     tree_node::{TreeNodeRecursion, TreeNodeVisitor},
     DFSchema, ParamValues, ScalarValue, SchemaReference, TableReference,
 };
+pub use datafusion_execution::config::SessionConfig;
 use datafusion_execution::registry::SerializerRegistry;
+pub use datafusion_execution::TaskContext;
+pub use datafusion_expr::execution_props::ExecutionProps;
 use datafusion_expr::{
     expr_rewriter::FunctionRewrite,
     logical_plan::{DdlStatement, Statement},
     planner::ExprPlanner,
     Expr, UserDefinedLogicalNode, WindowUDF,
 };
+use datafusion_optimizer::analyzer::type_coercion::TypeCoercion;
+use datafusion_optimizer::Analyzer;
+use datafusion_optimizer::{AnalyzerRule, OptimizerRule};
+use datafusion_session::SessionStore;
 
-// backwards compatibility
-pub use crate::execution::session_state::SessionState;
-
-use crate::datasource::dynamic_file::DynamicListTableFactory;
-use crate::execution::session_state::SessionStateBuilder;
 use async_trait::async_trait;
 use chrono::{DateTime, Utc};
-use datafusion_catalog::{
-    DynamicFileCatalog, SessionStore, TableFunction, TableFunctionImpl, UrlTableFactory,
-};
-pub use datafusion_execution::config::SessionConfig;
-pub use datafusion_execution::TaskContext;
-pub use datafusion_expr::execution_props::ExecutionProps;
-use datafusion_optimizer::{AnalyzerRule, OptimizerRule};
 use object_store::ObjectStore;
 use parking_lot::RwLock;
 use url::Url;
 
-mod avro;
 mod csv;
 mod json;
 #[cfg(feature = "parquet")]
 mod parquet;
 
+#[cfg(feature = "avro")]
+mod avro;
+
 /// DataFilePaths adds a method to convert strings and vector of strings to vector of [`ListingTableUrl`] URLs.
 /// This allows methods such [`SessionContext::read_csv`] and [`SessionContext::read_avro`]
 /// to take either a single file or multiple files.
@@ -854,6 +860,16 @@ impl SessionContext {
         }
     }
 
+    /// Applies the `TypeCoercion` rewriter to the logical plan.
+    fn apply_type_coercion(logical_plan: LogicalPlan) -> Result<LogicalPlan> {
+        let options = ConfigOptions::default();
+        Analyzer::with_rules(vec![Arc::new(TypeCoercion::new())]).execute_and_check(
+            logical_plan,
+            &options,
+            |_, _| {},
+        )
+    }
+
     async fn create_view(&self, cmd: CreateView) -> Result<DataFrame> {
         let CreateView {
             name,
@@ -872,13 +888,14 @@ impl SessionContext {
         match (or_replace, view) {
             (true, Ok(_)) => {
                 self.deregister_table(name.clone())?;
-                let table = Arc::new(ViewTable::try_new((*input).clone(), definition)?);
-
+                let input = Self::apply_type_coercion(input.as_ref().clone())?;
+                let table = Arc::new(ViewTable::new(input, definition));
                 self.register_table(name, table)?;
                 self.return_empty_dataframe()
             }
             (_, Err(_)) => {
-                let table = Arc::new(ViewTable::try_new((*input).clone(), definition)?);
+                let input = Self::apply_type_coercion(input.as_ref().clone())?;
+                let table = Arc::new(ViewTable::new(input, definition));
                 self.register_table(name, table)?;
                 self.return_empty_dataframe()
             }
@@ -1812,7 +1829,6 @@ impl<'n> TreeNodeVisitor<'n> for BadPlanVisitor<'_> {
 #[cfg(test)]
 mod tests {
     use super::{super::options::CsvReadOptions, *};
-    use crate::assert_batches_eq;
     use crate::execution::memory_pool::MemoryConsumer;
     use crate::test;
     use crate::test_util::{plan_and_collect, populate_csv_partitions};
@@ -1821,7 +1837,9 @@ mod tests {
     use std::error::Error;
     use std::path::PathBuf;
 
+    use datafusion_common::test_util::batches_to_string;
     use datafusion_common_runtime::SpawnedTask;
+    use insta::{allow_duplicates, assert_snapshot};
 
     use crate::catalog::SchemaProvider;
     use crate::execution::session_state::SessionStateBuilder;
@@ -1884,14 +1902,13 @@ mod tests {
             plan_and_collect(&ctx, "SELECT @@version, @name, @integer + 1 FROM dual")
                 .await?;
 
-        let expected = [
-            "+----------------------+------------------------+---------------------+",
-            "| @@version            | @name                  | @integer + Int64(1) |",
-            "+----------------------+------------------------+---------------------+",
-            "| system-var-@@version | user-defined-var-@name | 42                  |",
-            "+----------------------+------------------------+---------------------+",
-        ];
-        assert_batches_eq!(expected, &results);
+        assert_snapshot!(batches_to_string(&results), @r"
+        +----------------------+------------------------+---------------------+
+        | @@version            | @name                  | @integer + Int64(1) |
+        +----------------------+------------------------+---------------------+
+        | system-var-@@version | user-defined-var-@name | 42                  |
+        +----------------------+------------------------+---------------------+
+        ");
 
         Ok(())
     }
@@ -1972,14 +1989,15 @@ mod tests {
         let actual = arrow::util::pretty::pretty_format_batches(&result)
             .unwrap()
             .to_string();
-        let expected = r#"+--------------------+
-| c_name             |
-+--------------------+
-| Customer#000000002 |
-| Customer#000000003 |
-| Customer#000000004 |
-+--------------------+"#;
-        assert_eq!(actual, expected);
+        assert_snapshot!(actual, @r"
+        +--------------------+
+        | c_name             |
+        +--------------------+
+        | Customer#000000002 |
+        | Customer#000000003 |
+        | Customer#000000004 |
+        +--------------------+
+        ");
 
         Ok(())
     }
@@ -2004,14 +2022,15 @@ mod tests {
         let actual = arrow::util::pretty::pretty_format_batches(&result)
             .unwrap()
             .to_string();
-        let expected = r#"+--------------------+
-| c_name             |
-+--------------------+
-| Customer#000000002 |
-| Customer#000000003 |
-| Customer#000000004 |
-+--------------------+"#;
-        assert_eq!(actual, expected);
+        assert_snapshot!(actual, @r"
+        +--------------------+
+        | c_name             |
+        +--------------------+
+        | Customer#000000002 |
+        | Customer#000000003 |
+        | Customer#000000004 |
+        +--------------------+
+        ");
 
         Ok(())
     }
@@ -2094,6 +2113,8 @@ mod tests {
             .unwrap();
         ctx.register_catalog("my_catalog", Arc::new(catalog));
 
+        let mut results = Vec::new();
+
         for table_ref in &["my_catalog.my_schema.test", "my_schema.test", "test"] {
             let result = plan_and_collect(
                 &ctx,
@@ -2102,14 +2123,18 @@ mod tests {
             .await
             .unwrap();
 
-            let expected = [
-                "+-------+",
-                "| count |",
-                "+-------+",
-                "| 1     |",
-                "+-------+",
-            ];
-            assert_batches_eq!(expected, &result);
+            results.push(result);
+        }
+        allow_duplicates! {
+            for result in &results {
+                assert_snapshot!(batches_to_string(result), @r"
+                +-------+
+                | count |
+                +-------+
+                | 1     |
+                +-------+
+                ");
+            }
         }
     }
 
@@ -2144,15 +2169,14 @@ mod tests {
         )
         .await?;
 
-        let expected = [
-            "+-----+-------+",
-            "| cat | total |",
-            "+-----+-------+",
-            "| a   | 1     |",
-            "| b   | 3     |",
-            "+-----+-------+",
-        ];
-        assert_batches_eq!(expected, &result);
+        assert_snapshot!(batches_to_string(&result), @r"
+        +-----+-------+
+        | cat | total |
+        +-----+-------+
+        | a   | 1     |
+        | b   | 3     |
+        +-----+-------+
+        ");
 
         Ok(())
     }
@@ -2241,14 +2265,13 @@ mod tests {
             .await?
             .collect()
             .await?;
-        let expected = [
-            "+-----------------------------+",
-            "| Utf8(\"2021-01-01 00:00:00\") |",
-            "+-----------------------------+",
-            "| 2021-01-01T00:00:00         |",
-            "+-----------------------------+",
-        ];
-        assert_batches_eq!(expected, &result);
+        assert_snapshot!(batches_to_string(&result), @r#"
+        +-----------------------------+
+        | Utf8("2021-01-01 00:00:00") |
+        +-----------------------------+
+        | 2021-01-01T00:00:00         |
+        +-----------------------------+
+        "#);
         Ok(())
     }
     #[test]
