@@ -24,7 +24,7 @@ use datafusion::datasource::file_format::parquet::ParquetFormat;
 use datafusion::datasource::file_format::FileFormat;
 use datafusion::datasource::listing::PartitionedFile;
 use datafusion::datasource::object_store::ObjectStoreUrl;
-use datafusion::datasource::physical_plan::{FileScanConfig, ParquetSource};
+use datafusion::datasource::physical_plan::ParquetSource;
 use datafusion::datasource::source::DataSourceExec;
 use datafusion::execution::context::SessionState;
 use datafusion::physical_plan::metrics::MetricValue;
@@ -35,6 +35,7 @@ use datafusion_expr::execution_props::ExecutionProps;
 use datafusion_expr::{col, lit, Expr};
 use datafusion_physical_expr::create_physical_expr;
 
+use datafusion_datasource::file_scan_config::FileScanConfigBuilder;
 use futures::StreamExt;
 use object_store::path::Path;
 use object_store::ObjectMeta;
@@ -51,7 +52,7 @@ async fn get_parquet_exec(state: &SessionState, filter: Expr) -> DataSourceExec
     let meta = ObjectMeta {
         location,
         last_modified: metadata.modified().map(chrono::DateTime::from).unwrap(),
-        size: metadata.len() as usize,
+        size: metadata.len(),
         e_tag: None,
         version: None,
     };
@@ -79,8 +80,9 @@ async fn get_parquet_exec(state: &SessionState, filter: Expr) -> DataSourceExec
             .with_predicate(Arc::clone(&schema), predicate)
             .with_enable_page_index(true),
     );
-    let base_config =
-        FileScanConfig::new(object_store_url, schema, source).with_file(partitioned_file);
+    let base_config = FileScanConfigBuilder::new(object_store_url, schema, source)
+        .with_file(partitioned_file)
+        .build();
 
     DataSourceExec::new(Arc::new(base_config))
 }
