@@ -924,6 +924,10 @@ impl DisplayAs for UnboundedExec {
                     self.batch_produce.is_none(),
                 )
             }
+            DisplayFormatType::TreeRender => {
+                // TODO: collect info
+                write!(f, "")
+            }
         }
     }
 }
@@ -1019,6 +1023,11 @@ impl DisplayAs for StatisticsExec {
                     self.stats.num_rows,
                 )
             }
+
+            DisplayFormatType::TreeRender => {
+                // TODO: collect info
+                write!(f, "")
+            }
         }
     }
 }
