@@ -30,6 +30,7 @@ use arrow::array::{
 };
 use arrow::datatypes::{Fields, Schema};
 
+use datafusion::common::test_util::batches_to_string;
 use datafusion::dataframe::DataFrame;
 use datafusion::datasource::MemTable;
 use datafusion::test_util::plan_and_collect;
@@ -39,7 +40,6 @@ use datafusion::{
         datatypes::{DataType, Field, Float64Type, TimeUnit, TimestampNanosecondType},
         record_batch::RecordBatch,
     },
-    assert_batches_eq,
     error::Result,
     logical_expr::{
         AccumulatorFactoryFunction, AggregateUDF, Signature, TypeSignature, Volatility,
@@ -48,7 +48,8 @@ use datafusion::{
     prelude::SessionContext,
     scalar::ScalarValue,
 };
-use datafusion_common::{assert_contains, cast::as_primitive_array, exec_err};
+use datafusion_common::assert_contains;
+use datafusion_common::{cast::as_primitive_array, exec_err};
 use datafusion_expr::{
     col, create_udaf, function::AccumulatorArgs, AggregateUDFImpl, GroupsAccumulator,
     LogicalPlanBuilder, SimpleAggregateUDF,
@@ -60,18 +61,20 @@ use datafusion_functions_aggregate::average::AvgAccumulator;
 async fn test_setup() {
     let TestContext { ctx, test_state: _ } = TestContext::new();
     let sql = "SELECT * from t order by time";
-    let expected = [
-        "+-------+----------------------------+",
-        "| value | time                       |",
-        "+-------+----------------------------+",
-        "| 2.0   | 1970-01-01T00:00:00.000002 |",
-        "| 3.0   | 1970-01-01T00:00:00.000003 |",
-        "| 1.0   | 1970-01-01T00:00:00.000004 |",
-        "| 5.0   | 1970-01-01T00:00:00.000005 |",
-        "| 5.0   | 1970-01-01T00:00:00.000005 |",
-        "+-------+----------------------------+",
-    ];
-    assert_batches_eq!(expected, &execute(&ctx, sql).await.unwrap());
+
+    let actual = execute(&ctx, sql).await.unwrap();
+
+    insta::assert_snapshot!(batches_to_string(&actual), @r###"
+    +-------+----------------------------+
+    | value | time                       |
+    +-------+----------------------------+
+    | 2.0   | 1970-01-01T00:00:00.000002 |
+    | 3.0   | 1970-01-01T00:00:00.000003 |
+    | 1.0   | 1970-01-01T00:00:00.000004 |
+    | 5.0   | 1970-01-01T00:00:00.000005 |
+    | 5.0   | 1970-01-01T00:00:00.000005 |
+    +-------+----------------------------+
+    "###);
 }
 
 /// Basic user defined aggregate
@@ -80,14 +83,17 @@ async fn test_udaf() {
     let TestContext { ctx, test_state } = TestContext::new();
     assert!(!test_state.update_batch());
     let sql = "SELECT time_sum(time) from t";
-    let expected = [
-        "+----------------------------+",
-        "| time_sum(t.time)           |",
-        "+----------------------------+",
-        "| 1970-01-01T00:00:00.000019 |",
-        "+----------------------------+",
-    ];
-    assert_batches_eq!(expected, &execute(&ctx, sql).await.unwrap());
+
+    let actual = execute(&ctx, sql).await.unwrap();
+
+    insta::assert_snapshot!(batches_to_string(&actual), @r###"
+    +----------------------------+
+    | time_sum(t.time)           |
+    +----------------------------+
+    | 1970-01-01T00:00:00.000019 |
+    +----------------------------+
+    "###);
+
     // normal aggregates call update_batch
     assert!(test_state.update_batch());
     assert!(!test_state.retract_batch());
@@ -98,18 +104,21 @@ async fn test_udaf() {
 async fn test_udaf_as_window() {
     let TestContext { ctx, test_state } = TestContext::new();
     let sql = "SELECT time_sum(time) OVER() as time_sum from t";
-    let expected = [
-        "+----------------------------+",
-        "| time_sum                   |",
-        "+----------------------------+",
-        "| 1970-01-01T00:00:00.000019 |",
-        "| 1970-01-01T00:00:00.000019 |",
-        "| 1970-01-01T00:00:00.000019 |",
-        "| 1970-01-01T00:00:00.000019 |",
-        "| 1970-01-01T00:00:00.000019 |",
-        "+----------------------------+",
-    ];
-    assert_batches_eq!(expected, &execute(&ctx, sql).await.unwrap());
+
+    let actual = execute(&ctx, sql).await.unwrap();
+
+    insta::assert_snapshot!(batches_to_string(&actual), @r###"
+    +----------------------------+
+    | time_sum                   |
+    +----------------------------+
+    | 1970-01-01T00:00:00.000019 |
+    | 1970-01-01T00:00:00.000019 |
+    | 1970-01-01T00:00:00.000019 |
+    | 1970-01-01T00:00:00.000019 |
+    | 1970-01-01T00:00:00.000019 |
+    +----------------------------+
+    "###);
+
     // aggregate over the entire window function call update_batch
     assert!(test_state.update_batch());
     assert!(!test_state.retract_batch());
@@ -120,18 +129,21 @@ async fn test_udaf_as_window() {
 async fn test_udaf_as_window_with_frame() {
     let TestContext { ctx, test_state } = TestContext::new();
     let sql = "SELECT time_sum(time) OVER(ORDER BY time ROWS BETWEEN 1 PRECEDING AND 1 FOLLOWING) as time_sum from t";
-    let expected = [
-        "+----------------------------+",
-        "| time_sum                   |",
-        "+----------------------------+",
-        "| 1970-01-01T00:00:00.000005 |",
-        "| 1970-01-01T00:00:00.000009 |",
-        "| 1970-01-01T00:00:00.000012 |",
-        "| 1970-01-01T00:00:00.000014 |",
-        "| 1970-01-01T00:00:00.000010 |",
-        "+----------------------------+",
-    ];
-    assert_batches_eq!(expected, &execute(&ctx, sql).await.unwrap());
+
+    let actual = execute(&ctx, sql).await.unwrap();
+
+    insta::assert_snapshot!(batches_to_string(&actual), @r###"
+    +----------------------------+
+    | time_sum                   |
+    +----------------------------+
+    | 1970-01-01T00:00:00.000005 |
+    | 1970-01-01T00:00:00.000009 |
+    | 1970-01-01T00:00:00.000012 |
+    | 1970-01-01T00:00:00.000014 |
+    | 1970-01-01T00:00:00.000010 |
+    +----------------------------+
+    "###);
+
     // user defined aggregates with window frame should be calling retract batch
     assert!(test_state.update_batch());
     assert!(test_state.retract_batch());
@@ -155,14 +167,16 @@ async fn test_udaf_as_window_with_frame_without_retract_batch() {
 async fn test_udaf_returning_struct() {
     let TestContext { ctx, test_state: _ } = TestContext::new();
     let sql = "SELECT first(value, time) from t";
-    let expected = [
-        "+------------------------------------------------+",
-        "| first(t.value,t.time)                          |",
-        "+------------------------------------------------+",
-        "| {value: 2.0, time: 1970-01-01T00:00:00.000002} |",
-        "+------------------------------------------------+",
-    ];
-    assert_batches_eq!(expected, &execute(&ctx, sql).await.unwrap());
+
+    let actual = execute(&ctx, sql).await.unwrap();
+
+    insta::assert_snapshot!(batches_to_string(&actual), @r###"
+    +------------------------------------------------+
+    | first(t.value,t.time)                          |
+    +------------------------------------------------+
+    | {value: 2.0, time: 1970-01-01T00:00:00.000002} |
+    +------------------------------------------------+
+    "###);
 }
 
 /// Demonstrate extracting the fields from a structure using a subquery
@@ -170,14 +184,16 @@ async fn test_udaf_returning_struct() {
 async fn test_udaf_returning_struct_subquery() {
     let TestContext { ctx, test_state: _ } = TestContext::new();
     let sql = "select sq.first['value'], sq.first['time'] from (SELECT first(value, time) as first from t) as sq";
-    let expected = [
-        "+-----------------+----------------------------+",
-        "| sq.first[value] | sq.first[time]             |",
-        "+-----------------+----------------------------+",
-        "| 2.0             | 1970-01-01T00:00:00.000002 |",
-        "+-----------------+----------------------------+",
-    ];
-    assert_batches_eq!(expected, &execute(&ctx, sql).await.unwrap());
+
+    let actual = execute(&ctx, sql).await.unwrap();
+
+    insta::assert_snapshot!(batches_to_string(&actual), @r###"
+    +-----------------+----------------------------+
+    | sq.first[value] | sq.first[time]             |
+    +-----------------+----------------------------+
+    | 2.0             | 1970-01-01T00:00:00.000002 |
+    +-----------------+----------------------------+
+    "###);
 }
 
 #[tokio::test]
@@ -189,26 +205,29 @@ async fn test_udaf_shadows_builtin_fn() {
     let sql = "SELECT sum(arrow_cast(time, 'Int64')) from t";
 
     // compute with builtin `sum` aggregator
-    let expected = [
-        "+---------------------------------------+",
-        "| sum(arrow_cast(t.time,Utf8(\"Int64\"))) |",
-        "+---------------------------------------+",
-        "| 19000                                 |",
-        "+---------------------------------------+",
-    ];
-    assert_batches_eq!(expected, &execute(&ctx, sql).await.unwrap());
+    let actual = execute(&ctx, sql).await.unwrap();
+
+    insta::assert_snapshot!(batches_to_string(&actual), @r###"
+    +---------------------------------------+
+    | sum(arrow_cast(t.time,Utf8("Int64"))) |
+    +---------------------------------------+
+    | 19000                                 |
+    +---------------------------------------+
+    "###);
 
     // Register `TimeSum` with name `sum`. This will shadow the builtin one
-    let sql = "SELECT sum(time) from t";
     TimeSum::register(&mut ctx, test_state.clone(), "sum");
-    let expected = [
-        "+----------------------------+",
-        "| sum(t.time)                |",
-        "+----------------------------+",
-        "| 1970-01-01T00:00:00.000019 |",
-        "+----------------------------+",
-    ];
-    assert_batches_eq!(expected, &execute(&ctx, sql).await.unwrap());
+    let sql = "SELECT sum(time) from t";
+
+    let actual = execute(&ctx, sql).await.unwrap();
+
+    insta::assert_snapshot!(batches_to_string(&actual), @r###"
+    +----------------------------+
+    | sum(t.time)                |
+    +----------------------------+
+    | 1970-01-01T00:00:00.000019 |
+    +----------------------------+
+    "###);
 }
 
 async fn execute(ctx: &SessionContext, sql: &str) -> Result<Vec<RecordBatch>> {
@@ -248,14 +267,13 @@ async fn simple_udaf() -> Result<()> {
 
     let result = ctx.sql("SELECT MY_AVG(a) FROM t").await?.collect().await?;
 
-    let expected = [
-        "+-------------+",
-        "| my_avg(t.a) |",
-        "+-------------+",
-        "| 3.0         |",
-        "+-------------+",
-    ];
-    assert_batches_eq!(expected, &result);
+    insta::assert_snapshot!(batches_to_string(&result), @r###"
+    +-------------+
+    | my_avg(t.a) |
+    +-------------+
+    | 3.0         |
+    +-------------+
+    "###);
 
     Ok(())
 }
@@ -315,14 +333,13 @@ async fn case_sensitive_identifiers_user_defined_aggregates() -> Result<()> {
         .collect()
         .await?;
 
-    let expected = [
-        "+-------------+",
-        "| MY_AVG(t.i) |",
-        "+-------------+",
-        "| 1.0         |",
-        "+-------------+",
-    ];
-    assert_batches_eq!(expected, &result);
+    insta::assert_snapshot!(batches_to_string(&result), @r###"
+    +-------------+
+    | MY_AVG(t.i) |
+    +-------------+
+    | 1.0         |
+    +-------------+
+    "###);
 
     Ok(())
 }
@@ -346,19 +363,25 @@ async fn test_user_defined_functions_with_alias() -> Result<()> {
 
     ctx.register_udaf(my_avg);
 
-    let expected = [
-        "+------------+",
-        "| dummy(t.i) |",
-        "+------------+",
-        "| 1.0        |",
-        "+------------+",
-    ];
-
     let result = plan_and_collect(&ctx, "SELECT dummy(i) FROM t").await?;
-    assert_batches_eq!(expected, &result);
+
+    insta::assert_snapshot!(batches_to_string(&result), @r###"
+    +------------+
+    | dummy(t.i) |
+    +------------+
+    | 1.0        |
+    +------------+
+    "###);
 
     let alias_result = plan_and_collect(&ctx, "SELECT dummy_alias(i) FROM t").await?;
-    assert_batches_eq!(expected, &alias_result);
+
+    insta::assert_snapshot!(batches_to_string(&alias_result), @r###"
+    +------------+
+    | dummy(t.i) |
+    +------------+
+    | 1.0        |
+    +------------+
+    "###);
 
     Ok(())
 }
@@ -418,14 +441,14 @@ async fn test_parameterized_aggregate_udf() -> Result<()> {
     );
 
     let actual = DataFrame::new(ctx.state(), plan).collect().await?;
-    let expected = [
-        "+------+---+---+",
-        "| text | a | b |",
-        "+------+---+---+",
-        "| foo  | 1 | 2 |",
-        "+------+---+---+",
-    ];
-    assert_batches_eq!(expected, &actual);
+
+    insta::assert_snapshot!(batches_to_string(&actual), @r###"
+    +------+---+---+
+    | text | a | b |
+    +------+---+---+
+    | foo  | 1 | 2 |
+    +------+---+---+
+    "###);
 
     ctx.deregister_table("t")?;
     Ok(())
