@@ -27,7 +27,7 @@ use crate::{
 };
 
 use crate::cache::cache_manager::{CacheManager, CacheManagerConfig};
-use datafusion_common::{DataFusionError, Result};
+use datafusion_common::Result;
 use object_store::ObjectStore;
 use std::path::PathBuf;
 use std::sync::Arc;
@@ -150,9 +150,7 @@ impl RuntimeEnv {
     /// registry. See [`ObjectStoreRegistry::get_store`] for more
     /// details.
     pub fn object_store(&self, url: impl AsRef<Url>) -> Result<Arc<dyn ObjectStore>> {
-        self.object_store_registry
-            .get_store(url.as_ref())
-            .map_err(DataFusionError::from)
+        self.object_store_registry.get_store(url.as_ref())
     }
 }
 
