@@ -30,8 +30,8 @@ use arrow::datatypes::{
 };
 use datafusion_common::types::NativeType;
 use datafusion_common::{
-    exec_err, internal_err, plan_datafusion_err, plan_err, Diagnostic, Result, Span,
-    Spans,
+    exec_err, internal_err, not_impl_err, plan_datafusion_err, plan_err, Diagnostic,
+    Result, Span, Spans,
 };
 use itertools::Itertools;
 
@@ -182,16 +182,23 @@ impl<'a> BinaryTypeCoercer<'a> {
             })
         }
         AtArrow | ArrowAt => {
-            // ArrowAt and AtArrow check for whether one array is contained in another.
-            // The result type is boolean. Signature::comparison defines this signature.
-            // Operation has nothing to do with comparison
-            array_coercion(self.lhs, self.rhs).map(Signature::comparison).ok_or_else(|| {
+            // Array contains or search (similar to LIKE) operation
+            array_coercion(self.lhs, self.rhs)
+                .or_else(|| like_coercion(self.lhs, self.rhs)).map(Signature::comparison).ok_or_else(|| {
+                    plan_datafusion_err!(
+                        "Cannot infer common argument type for operation {} {} {}", self.lhs, self.op, self.rhs
+                    )
+                })
+        }
+        AtAt => {
+            // text search has similar signature to LIKE
+            like_coercion(self.lhs, self.rhs).map(Signature::comparison).ok_or_else(|| {
                 plan_datafusion_err!(
-                    "Cannot infer common array type for arrow operation {} {} {}", self.lhs, self.op, self.rhs
+                    "Cannot infer common argument type for AtAt operation {} {} {}", self.lhs, self.op, self.rhs
                 )
             })
         }
-        Plus | Minus | Multiply | Divide | Modulo =>  {
+        Plus | Minus | Multiply | Divide | Modulo  =>  {
             let get_result = |lhs, rhs| {
                 use arrow::compute::kernels::numeric::*;
                 let l = new_empty_array(lhs);
@@ -248,6 +255,10 @@ impl<'a> BinaryTypeCoercer<'a> {
                     "Cannot coerce arithmetic expression {} {} {} to valid types", self.lhs, self.op, self.rhs
                 )
             }
+        },
+        IntegerDivide | Arrow | LongArrow | HashArrow | HashLongArrow
+        | HashMinus | AtQuestion | Question | QuestionAnd | QuestionPipe => {
+            not_impl_err!("Operator {} is not yet supported", self.op)
         }
     };
         result.map_err(|err| {
@@ -708,7 +719,7 @@ pub fn try_type_union_resolution_with_struct(
 /// strings.  For example when comparing `'2' > 1`,  the arguments will be
 /// coerced to `Utf8` for comparison
 pub fn comparison_coercion(lhs_type: &DataType, rhs_type: &DataType) -> Option<DataType> {
-    if lhs_type == rhs_type {
+    if lhs_type.equals_datatype(rhs_type) {
         // same type => equality is possible
         return Some(lhs_type.clone());
     }
@@ -722,6 +733,7 @@ pub fn comparison_coercion(lhs_type: &DataType, rhs_type: &DataType) -> Option<D
         .or_else(|| string_temporal_coercion(lhs_type, rhs_type))
         .or_else(|| binary_coercion(lhs_type, rhs_type))
         .or_else(|| struct_coercion(lhs_type, rhs_type))
+        .or_else(|| map_coercion(lhs_type, rhs_type))
 }
 
 /// Similar to [`comparison_coercion`] but prefers numeric if compares with
@@ -827,7 +839,6 @@ pub fn binary_numeric_coercion(
     lhs_type: &DataType,
     rhs_type: &DataType,
 ) -> Option<DataType> {
-    use arrow::datatypes::DataType::*;
     if !lhs_type.is_numeric() || !rhs_type.is_numeric() {
         return None;
     };
@@ -841,39 +852,7 @@ pub fn binary_numeric_coercion(
         return Some(t);
     }
 
-    // These are ordered from most informative to least informative so
-    // that the coercion does not lose information via truncation
-    match (lhs_type, rhs_type) {
-        (Float64, _) | (_, Float64) => Some(Float64),
-        (_, Float32) | (Float32, _) => Some(Float32),
-        // The following match arms encode the following logic: Given the two
-        // integral types, we choose the narrowest possible integral type that
-        // accommodates all values of both types. Note that to avoid information
-        // loss when combining UInt64 with signed integers we use Decimal128(20, 0).
-        (UInt64, Int64 | Int32 | Int16 | Int8)
-        | (Int64 | Int32 | Int16 | Int8, UInt64) => Some(Decimal128(20, 0)),
-        (UInt64, _) | (_, UInt64) => Some(UInt64),
-        (Int64, _)
-        | (_, Int64)
-        | (UInt32, Int8)
-        | (Int8, UInt32)
-        | (UInt32, Int16)
-        | (Int16, UInt32)
-        | (UInt32, Int32)
-        | (Int32, UInt32) => Some(Int64),
-        (Int32, _)
-        | (_, Int32)
-        | (UInt16, Int16)
-        | (Int16, UInt16)
-        | (UInt16, Int8)
-        | (Int8, UInt16) => Some(Int32),
-        (UInt32, _) | (_, UInt32) => Some(UInt32),
-        (Int16, _) | (_, Int16) | (Int8, UInt8) | (UInt8, Int8) => Some(Int16),
-        (UInt16, _) | (_, UInt16) => Some(UInt16),
-        (Int8, _) | (_, Int8) => Some(Int8),
-        (UInt8, _) | (_, UInt8) => Some(UInt8),
-        _ => None,
-    }
+    numerical_coercion(lhs_type, rhs_type)
 }
 
 /// Decimal coercion rules.
@@ -1009,6 +988,25 @@ fn coerce_fields(common_type: DataType, lhs: &FieldRef, rhs: &FieldRef) -> Field
     Arc::new(Field::new(name, common_type, is_nullable))
 }
 
+/// coerce two types if they are Maps by coercing their inner 'entries' fields' types
+/// using struct coercion
+fn map_coercion(lhs_type: &DataType, rhs_type: &DataType) -> Option<DataType> {
+    use arrow::datatypes::DataType::*;
+    match (lhs_type, rhs_type) {
+        (Map(lhs_field, lhs_ordered), Map(rhs_field, rhs_ordered)) => {
+            struct_coercion(lhs_field.data_type(), rhs_field.data_type()).map(
+                |key_value_type| {
+                    Map(
+                        Arc::new((**lhs_field).clone().with_data_type(key_value_type)),
+                        *lhs_ordered && *rhs_ordered,
+                    )
+                },
+            )
+        }
+        _ => None,
+    }
+}
+
 /// Returns the output type of applying mathematics operations such as
 /// `+` to arguments of `lhs_type` and `rhs_type`.
 fn mathematics_numerical_coercion(
@@ -1034,15 +1032,36 @@ fn mathematics_numerical_coercion(
         (_, Dictionary(_, value_type)) => {
             mathematics_numerical_coercion(lhs_type, value_type)
         }
+        _ => numerical_coercion(lhs_type, rhs_type),
+    }
+}
+
+/// A common set of numerical coercions that are applied for mathematical and binary ops
+/// to `lhs_type` and `rhs_type`.
+fn numerical_coercion(lhs_type: &DataType, rhs_type: &DataType) -> Option<DataType> {
+    use arrow::datatypes::DataType::*;
+
+    match (lhs_type, rhs_type) {
         (Float64, _) | (_, Float64) => Some(Float64),
         (_, Float32) | (Float32, _) => Some(Float32),
-        (Int64, _) | (_, Int64) => Some(Int64),
-        (Int32, _) | (_, Int32) => Some(Int32),
-        (Int16, _) | (_, Int16) => Some(Int16),
-        (Int8, _) | (_, Int8) => Some(Int8),
+        // The following match arms encode the following logic: Given the two
+        // integral types, we choose the narrowest possible integral type that
+        // accommodates all values of both types. Note that to avoid information
+        // loss when combining UInt64 with signed integers we use Decimal128(20, 0).
+        (UInt64, Int64 | Int32 | Int16 | Int8)
+        | (Int64 | Int32 | Int16 | Int8, UInt64) => Some(Decimal128(20, 0)),
         (UInt64, _) | (_, UInt64) => Some(UInt64),
+        (Int64, _)
+        | (_, Int64)
+        | (UInt32, Int32 | Int16 | Int8)
+        | (Int32 | Int16 | Int8, UInt32) => Some(Int64),
         (UInt32, _) | (_, UInt32) => Some(UInt32),
+        (Int32, _) | (_, Int32) | (UInt16, Int16 | Int8) | (Int16 | Int8, UInt16) => {
+            Some(Int32)
+        }
         (UInt16, _) | (_, UInt16) => Some(UInt16),
+        (Int16, _) | (_, Int16) | (Int8, UInt8) | (UInt8, Int8) => Some(Int16),
+        (Int8, _) | (_, Int8) => Some(Int8),
         (UInt8, _) | (_, UInt8) => Some(UInt8),
         _ => None,
     }
@@ -1177,26 +1196,6 @@ pub fn string_coercion(lhs_type: &DataType, rhs_type: &DataType) -> Option<DataT
     }
 }
 
-/// This will be deprecated when binary operators native support
-/// for Utf8View (use `string_coercion` instead).
-fn regex_comparison_string_coercion(
-    lhs_type: &DataType,
-    rhs_type: &DataType,
-) -> Option<DataType> {
-    use arrow::datatypes::DataType::*;
-    match (lhs_type, rhs_type) {
-        // If Utf8View is in any side, we coerce to Utf8.
-        (Utf8View, Utf8View | Utf8 | LargeUtf8) | (Utf8 | LargeUtf8, Utf8View) => {
-            Some(Utf8)
-        }
-        // Then, if LargeUtf8 is in any side, we coerce to LargeUtf8.
-        (LargeUtf8, Utf8 | LargeUtf8) | (Utf8, LargeUtf8) => Some(LargeUtf8),
-        // Utf8 coerces to Utf8
-        (Utf8, Utf8) => Some(Utf8),
-        _ => None,
-    }
-}
-
 fn numeric_string_coercion(lhs_type: &DataType, rhs_type: &DataType) -> Option<DataType> {
     use arrow::datatypes::DataType::*;
     match (lhs_type, rhs_type) {
@@ -1327,7 +1326,7 @@ fn regex_null_coercion(lhs_type: &DataType, rhs_type: &DataType) -> Option<DataT
 /// Coercion rules for regular expression comparison operations.
 /// This is a union of string coercion rules and dictionary coercion rules
 pub fn regex_coercion(lhs_type: &DataType, rhs_type: &DataType) -> Option<DataType> {
-    regex_comparison_string_coercion(lhs_type, rhs_type)
+    string_coercion(lhs_type, rhs_type)
         .or_else(|| dictionary_comparison_coercion(lhs_type, rhs_type, false))
         .or_else(|| regex_null_coercion(lhs_type, rhs_type))
 }
@@ -1641,7 +1640,7 @@ mod tests {
 
     /// Test coercion rules for binary operators
     ///
-    /// Applies coercion rules for `$LHS_TYPE $OP $RHS_TYPE` and asserts that the
+    /// Applies coercion rules for `$LHS_TYPE $OP $RHS_TYPE` and asserts that
     /// the result type is `$RESULT_TYPE`
     macro_rules! test_coercion_binary_rule {
         ($LHS_TYPE:expr, $RHS_TYPE:expr, $OP:expr, $RESULT_TYPE:expr) => {{
@@ -1652,6 +1651,26 @@ mod tests {
         }};
     }
 
+    /// Test coercion rules for binary operators
+    ///
+    /// Applies coercion rules for each RHS_TYPE in $RHS_TYPES such that
+    /// `$LHS_TYPE $OP RHS_TYPE` and asserts that the result type is `$RESULT_TYPE`.
+    /// Also tests that the inverse `RHS_TYPE $OP $LHS_TYPE` is true
+    macro_rules! test_coercion_binary_rule_multiple {
+        ($LHS_TYPE:expr, $RHS_TYPES:expr, $OP:expr, $RESULT_TYPE:expr) => {{
+            for rh_type in $RHS_TYPES {
+                let (lhs, rhs) = BinaryTypeCoercer::new(&$LHS_TYPE, &$OP, &rh_type)
+                    .get_input_types()?;
+                assert_eq!(lhs, $RESULT_TYPE);
+                assert_eq!(rhs, $RESULT_TYPE);
+
+                BinaryTypeCoercer::new(&rh_type, &$OP, &$LHS_TYPE).get_input_types()?;
+                assert_eq!(lhs, $RESULT_TYPE);
+                assert_eq!(rhs, $RESULT_TYPE);
+            }
+        }};
+    }
+
     /// Test coercion rules for like
     ///
     /// Applies coercion rules for both
@@ -1802,42 +1821,168 @@ mod tests {
             Operator::RegexMatch,
             DataType::Utf8
         );
+        test_coercion_binary_rule!(
+            DataType::Utf8,
+            DataType::Utf8View,
+            Operator::RegexMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Utf8View,
+            DataType::Utf8,
+            Operator::RegexMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Utf8View,
+            DataType::Utf8View,
+            Operator::RegexMatch,
+            DataType::Utf8View
+        );
         test_coercion_binary_rule!(
             DataType::Utf8,
             DataType::Utf8,
             Operator::RegexNotMatch,
             DataType::Utf8
         );
+        test_coercion_binary_rule!(
+            DataType::Utf8View,
+            DataType::Utf8,
+            Operator::RegexNotMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Utf8,
+            DataType::Utf8View,
+            Operator::RegexNotMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Utf8View,
+            DataType::Utf8View,
+            Operator::RegexNotMatch,
+            DataType::Utf8View
+        );
         test_coercion_binary_rule!(
             DataType::Utf8,
             DataType::Utf8,
             Operator::RegexNotIMatch,
             DataType::Utf8
         );
+        test_coercion_binary_rule!(
+            DataType::Utf8View,
+            DataType::Utf8,
+            Operator::RegexNotIMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Utf8,
+            DataType::Utf8View,
+            Operator::RegexNotIMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Utf8View,
+            DataType::Utf8View,
+            Operator::RegexNotIMatch,
+            DataType::Utf8View
+        );
         test_coercion_binary_rule!(
             DataType::Dictionary(DataType::Int32.into(), DataType::Utf8.into()),
             DataType::Utf8,
             Operator::RegexMatch,
             DataType::Utf8
         );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8.into()),
+            DataType::Utf8View,
+            Operator::RegexMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8View.into()),
+            DataType::Utf8,
+            Operator::RegexMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8View.into()),
+            DataType::Utf8View,
+            Operator::RegexMatch,
+            DataType::Utf8View
+        );
         test_coercion_binary_rule!(
             DataType::Dictionary(DataType::Int32.into(), DataType::Utf8.into()),
             DataType::Utf8,
             Operator::RegexIMatch,
             DataType::Utf8
         );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8View.into()),
+            DataType::Utf8,
+            Operator::RegexIMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8.into()),
+            DataType::Utf8View,
+            Operator::RegexIMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8View.into()),
+            DataType::Utf8View,
+            Operator::RegexIMatch,
+            DataType::Utf8View
+        );
         test_coercion_binary_rule!(
             DataType::Dictionary(DataType::Int32.into(), DataType::Utf8.into()),
             DataType::Utf8,
             Operator::RegexNotMatch,
             DataType::Utf8
         );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8.into()),
+            DataType::Utf8View,
+            Operator::RegexNotMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8View.into()),
+            DataType::Utf8,
+            Operator::RegexNotMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8.into()),
+            DataType::Utf8View,
+            Operator::RegexNotMatch,
+            DataType::Utf8View
+        );
         test_coercion_binary_rule!(
             DataType::Dictionary(DataType::Int32.into(), DataType::Utf8.into()),
             DataType::Utf8,
             Operator::RegexNotIMatch,
             DataType::Utf8
         );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8View.into()),
+            DataType::Utf8,
+            Operator::RegexNotIMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8.into()),
+            DataType::Utf8View,
+            Operator::RegexNotIMatch,
+            DataType::Utf8View
+        );
+        test_coercion_binary_rule!(
+            DataType::Dictionary(DataType::Int32.into(), DataType::Utf8View.into()),
+            DataType::Utf8View,
+            Operator::RegexNotIMatch,
+            DataType::Utf8View
+        );
         test_coercion_binary_rule!(
             DataType::Int16,
             DataType::Int64,
@@ -1885,39 +2030,94 @@ mod tests {
 
     #[test]
     fn test_type_coercion_arithmetic() -> Result<()> {
-        // integer
-        test_coercion_binary_rule!(
-            DataType::Int32,
-            DataType::UInt32,
+        use DataType::*;
+
+        // (Float64, _) | (_, Float64) => Some(Float64),
+        test_coercion_binary_rule_multiple!(
+            Float64,
+            [
+                Float64, Float32, Float16, Int64, UInt64, Int32, UInt32, Int16, UInt16,
+                Int8, UInt8
+            ],
             Operator::Plus,
-            DataType::Int32
+            Float64
         );
-        test_coercion_binary_rule!(
-            DataType::Int32,
-            DataType::UInt16,
+        // (_, Float32) | (Float32, _) => Some(Float32),
+        test_coercion_binary_rule_multiple!(
+            Float32,
+            [
+                Float32, Float16, Int64, UInt64, Int32, UInt32, Int16, UInt16, Int8,
+                UInt8
+            ],
+            Operator::Plus,
+            Float32
+        );
+        // (UInt64, Int64 | Int32 | Int16 | Int8) | (Int64 | Int32 | Int16 | Int8, UInt64)  => Some(Decimal128(20, 0)),
+        test_coercion_binary_rule_multiple!(
+            UInt64,
+            [Int64, Int32, Int16, Int8],
+            Operator::Divide,
+            Decimal128(20, 0)
+        );
+        // (UInt64, _) | (_, UInt64) => Some(UInt64),
+        test_coercion_binary_rule_multiple!(
+            UInt64,
+            [UInt64, UInt32, UInt16, UInt8],
+            Operator::Modulo,
+            UInt64
+        );
+        // (Int64, _) | (_, Int64) => Some(Int64),
+        test_coercion_binary_rule_multiple!(
+            Int64,
+            [Int64, Int32, UInt32, Int16, UInt16, Int8, UInt8],
+            Operator::Modulo,
+            Int64
+        );
+        // (UInt32, Int32 | Int16 | Int8) | (Int32 | Int16 | Int8, UInt32) => Some(Int64)
+        test_coercion_binary_rule_multiple!(
+            UInt32,
+            [Int32, Int16, Int8],
+            Operator::Modulo,
+            Int64
+        );
+        // (UInt32, _) | (_, UInt32) => Some(UInt32),
+        test_coercion_binary_rule_multiple!(
+            UInt32,
+            [UInt32, UInt16, UInt8],
+            Operator::Modulo,
+            UInt32
+        );
+        // (Int32, _) | (_, Int32) => Some(Int32),
+        test_coercion_binary_rule_multiple!(
+            Int32,
+            [Int32, Int16, Int8],
+            Operator::Modulo,
+            Int32
+        );
+        // (UInt16, Int16 | Int8) | (Int16 | Int8, UInt16) => Some(Int32)
+        test_coercion_binary_rule_multiple!(
+            UInt16,
+            [Int16, Int8],
             Operator::Minus,
-            DataType::Int32
+            Int32
         );
-        test_coercion_binary_rule!(
-            DataType::Int8,
-            DataType::Int64,
-            Operator::Multiply,
-            DataType::Int64
-        );
-        // float
-        test_coercion_binary_rule!(
-            DataType::Float32,
-            DataType::Int32,
+        // (UInt16, _) | (_, UInt16) => Some(UInt16),
+        test_coercion_binary_rule_multiple!(
+            UInt16,
+            [UInt16, UInt8, UInt8],
             Operator::Plus,
-            DataType::Float32
+            UInt16
         );
-        test_coercion_binary_rule!(
-            DataType::Float32,
-            DataType::Float64,
-            Operator::Multiply,
-            DataType::Float64
-        );
-        // TODO add other data type
+        // (Int16, _) | (_, Int16) => Some(Int16),
+        test_coercion_binary_rule_multiple!(Int16, [Int16, Int8], Operator::Plus, Int16);
+        // (UInt8, Int8) | (Int8, UInt8) => Some(Int16)
+        test_coercion_binary_rule!(Int8, UInt8, Operator::Minus, Int16);
+        test_coercion_binary_rule!(UInt8, Int8, Operator::Multiply, Int16);
+        // (UInt8, _) | (_, UInt8) => Some(UInt8),
+        test_coercion_binary_rule!(UInt8, UInt8, Operator::Minus, UInt8);
+        // (Int8, _) | (_, Int8) => Some(Int8),
+        test_coercion_binary_rule!(Int8, Int8, Operator::Plus, Int8);
+
         Ok(())
     }
 
@@ -2303,4 +2503,49 @@ mod tests {
         );
         Ok(())
     }
+
+    #[test]
+    fn test_map_coercion() -> Result<()> {
+        let lhs = Field::new_map(
+            "lhs",
+            "entries",
+            Arc::new(Field::new("keys", DataType::Utf8, false)),
+            Arc::new(Field::new("values", DataType::LargeUtf8, false)),
+            true,
+            false,
+        );
+        let rhs = Field::new_map(
+            "rhs",
+            "kvp",
+            Arc::new(Field::new("k", DataType::Utf8, false)),
+            Arc::new(Field::new("v", DataType::Utf8, true)),
+            false,
+            true,
+        );
+
+        let expected = Field::new_map(
+            "expected",
+            "entries", // struct coercion takes lhs name
+            Arc::new(Field::new(
+                "keys", // struct coercion takes lhs name
+                DataType::Utf8,
+                false,
+            )),
+            Arc::new(Field::new(
+                "values",            // struct coercion takes lhs name
+                DataType::LargeUtf8, // lhs is large string
+                true,                // rhs is nullable
+            )),
+            false, // both sides must be sorted
+            true,  // rhs is nullable
+        );
+
+        test_coercion_binary_rule!(
+            lhs.data_type(),
+            rhs.data_type(),
+            Operator::Eq,
+            expected.data_type().clone()
+        );
+        Ok(())
+    }
 }
