@@ -15,8 +15,13 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use crate::udf::FFI_ScalarUDF;
-use datafusion::{functions::math::abs::AbsFunc, logical_expr::ScalarUDF};
+use crate::{udf::FFI_ScalarUDF, udtf::FFI_TableFunction};
+use datafusion::{
+    catalog::TableFunctionImpl,
+    functions::math::{abs::AbsFunc, random::RandomFunc},
+    functions_table::generate_series::RangeFunc,
+    logical_expr::ScalarUDF,
+};
 
 use std::sync::Arc;
 
@@ -25,3 +30,15 @@ pub(crate) extern "C" fn create_ffi_abs_func() -> FFI_ScalarUDF {
 
     udf.into()
 }
+
+pub(crate) extern "C" fn create_ffi_random_func() -> FFI_ScalarUDF {
+    let udf: Arc<ScalarUDF> = Arc::new(RandomFunc::new().into());
+
+    udf.into()
+}
+
+pub(crate) extern "C" fn create_ffi_table_func() -> FFI_TableFunction {
+    let udtf: Arc<dyn TableFunctionImpl> = Arc::new(RangeFunc {});
+
+    FFI_TableFunction::new(udtf, None)
+}
