@@ -18,7 +18,7 @@
 use std::any::Any;
 use std::fmt::{Debug, Formatter};
 use std::mem::size_of_val;
-use std::sync::{Arc, OnceLock};
+use std::sync::Arc;
 
 use arrow::{
     array::ArrayRef,
@@ -27,7 +27,6 @@ use arrow::{
 
 use datafusion_common::ScalarValue;
 use datafusion_common::{not_impl_err, plan_err, Result};
-use datafusion_expr::aggregate_doc_sections::DOC_SECTION_APPROXIMATE;
 use datafusion_expr::function::{AccumulatorArgs, StateFieldsArgs};
 use datafusion_expr::type_coercion::aggregates::NUMERICS;
 use datafusion_expr::Volatility::Immutable;
@@ -37,6 +36,7 @@ use datafusion_expr::{
 use datafusion_functions_aggregate_common::tdigest::{
     Centroid, TDigest, DEFAULT_MAX_SIZE,
 };
+use datafusion_macros::user_doc;
 
 use crate::approx_percentile_cont::{ApproxPercentileAccumulator, ApproxPercentileCont};
 
@@ -49,6 +49,28 @@ make_udaf_expr_and_func!(
 );
 
 /// APPROX_PERCENTILE_CONT_WITH_WEIGHT aggregate expression
+#[user_doc(
+    doc_section(label = "Approximate Functions"),
+    description = "Returns the weighted approximate percentile of input values using the t-digest algorithm.",
+    syntax_example = "approx_percentile_cont_with_weight(expression, weight, percentile)",
+    sql_example = r#"```sql
+> SELECT approx_percentile_cont_with_weight(column_name, weight_column, 0.90) FROM table_name;
++----------------------------------------------------------------------+
+| approx_percentile_cont_with_weight(column_name, weight_column, 0.90) |
++----------------------------------------------------------------------+
+| 78.5                                                                 |
++----------------------------------------------------------------------+
+```"#,
+    standard_argument(name = "expression", prefix = "The"),
+    argument(
+        name = "weight",
+        description = "Expression to use as weight. Can be a constant, column, or function, and any combination of arithmetic operators."
+    ),
+    argument(
+        name = "percentile",
+        description = "Percentile to compute. Must be a float value between 0 and 1 (inclusive)."
+    )
+)]
 pub struct ApproxPercentileContWithWeight {
     signature: Signature,
     approx_percentile_cont: ApproxPercentileCont,
@@ -150,44 +172,17 @@ impl AggregateUDFImpl for ApproxPercentileContWithWeight {
     }
 
     #[allow(rustdoc::private_intra_doc_links)]
-    /// See [`TDigest::to_scalar_state()`] for a description of the serialised
+    /// See [`TDigest::to_scalar_state()`] for a description of the serialized
     /// state.
     fn state_fields(&self, args: StateFieldsArgs) -> Result<Vec<Field>> {
         self.approx_percentile_cont.state_fields(args)
     }
 
     fn documentation(&self) -> Option<&Documentation> {
-        Some(get_approx_percentile_cont_with_weight_doc())
+        self.doc()
     }
 }
 
-static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
-
-fn get_approx_percentile_cont_with_weight_doc() -> &'static Documentation {
-    DOCUMENTATION.get_or_init(|| {
-        Documentation::builder()
-            .with_doc_section(DOC_SECTION_APPROXIMATE)
-            .with_description(
-                "Returns the weighted approximate percentile of input values using the t-digest algorithm.",
-            )
-            .with_syntax_example("approx_percentile_cont_with_weight(expression, weight, percentile)")
-            .with_sql_example(r#"```sql
-> SELECT approx_percentile_cont_with_weight(column_name, weight_column, 0.90) FROM table_name;
-+----------------------------------------------------------------------+
-| approx_percentile_cont_with_weight(column_name, weight_column, 0.90) |
-+----------------------------------------------------------------------+
-| 78.5                                                                 |
-+----------------------------------------------------------------------+
-```"#,
-            )
-            .with_standard_argument("expression", None)
-            .with_argument("weight", "Expression to use as weight. Can be a constant, column, or function, and any combination of arithmetic operators.")
-            .with_argument("percentile", "Percentile to compute. Must be a float value between 0 and 1 (inclusive).")
-            .build()
-            .unwrap()
-    })
-}
-
 #[derive(Debug)]
 pub struct ApproxPercentileWithWeightAccumulator {
     approx_percentile_cont_accumulator: ApproxPercentileAccumulator,
