@@ -19,18 +19,17 @@
 
 use std::any::Any;
 use std::fmt;
-use std::sync::OnceLock;
 
 use arrow::datatypes::DataType;
 use arrow::datatypes::Field;
 use datafusion_common::{not_impl_err, Result};
-use datafusion_expr::aggregate_doc_sections::DOC_SECTION_GENERAL;
 use datafusion_expr::function::AccumulatorArgs;
 use datafusion_expr::function::StateFieldsArgs;
 use datafusion_expr::utils::format_state_name;
 use datafusion_expr::{
     Accumulator, AggregateUDFImpl, Documentation, Signature, Volatility,
 };
+use datafusion_macros::user_doc;
 
 make_udaf_expr_and_func!(
     Grouping,
@@ -40,6 +39,27 @@ make_udaf_expr_and_func!(
     grouping_udaf
 );
 
+#[user_doc(
+    doc_section(label = "General Functions"),
+    description = "Returns 1 if the data is aggregated across the specified column, or 0 if it is not aggregated in the result set.",
+    syntax_example = "grouping(expression)",
+    sql_example = r#"```sql
+> SELECT column_name, GROUPING(column_name) AS group_column
+  FROM table_name
+  GROUP BY GROUPING SETS ((column_name), ());
++-------------+-------------+
+| column_name | group_column |
++-------------+-------------+
+| value1      | 0           |
+| value2      | 0           |
+| NULL        | 1           |
++-------------+-------------+
+```"#,
+    argument(
+        name = "expression",
+        description = "Expression to evaluate whether data is aggregated across the specified column. Can be a constant, column, or function."
+    )
+)]
 pub struct Grouping {
     signature: Signature,
 }
@@ -100,35 +120,6 @@ impl AggregateUDFImpl for Grouping {
     }
 
     fn documentation(&self) -> Option<&Documentation> {
-        Some(get_grouping_doc())
+        self.doc()
     }
 }
-
-static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
-
-fn get_grouping_doc() -> &'static Documentation {
-    DOCUMENTATION.get_or_init(|| {
-        Documentation::builder()
-            .with_doc_section(DOC_SECTION_GENERAL)
-            .with_description(
-                "Returns 1 if the data is aggregated across the specified column, or 0 if it is not aggregated in the result set.",
-            )
-            .with_syntax_example("grouping(expression)")
-            .with_sql_example(r#"```sql
-> SELECT column_name, GROUPING(column_name) AS group_column
-  FROM table_name
-  GROUP BY GROUPING SETS ((column_name), ());
-+-------------+-------------+
-| column_name | group_column |
-+-------------+-------------+
-| value1      | 0           |
-| value2      | 0           |
-| NULL        | 1           |
-+-------------+-------------+
-```"#, 
-            )
-            .with_argument("expression", "Expression to evaluate whether data is aggregated across the specified column. Can be a constant, column, or function.")
-            .build()
-            .unwrap()
-    })
-}
