@@ -18,16 +18,17 @@
 //! [`ScalarUDFImpl`] definitions for map_keys function.
 
 use crate::utils::{get_map_entry_field, make_scalar_function};
-use arrow_array::{Array, ArrayRef, ListArray};
-use arrow_schema::{DataType, Field};
+use arrow::array::{Array, ArrayRef, ListArray};
+use arrow::datatypes::{DataType, Field};
+use datafusion_common::utils::take_function_args;
 use datafusion_common::{cast::as_map_array, exec_err, Result};
-use datafusion_expr::scalar_doc_sections::DOC_SECTION_MAP;
 use datafusion_expr::{
     ArrayFunctionSignature, ColumnarValue, Documentation, ScalarUDFImpl, Signature,
     TypeSignature, Volatility,
 };
+use datafusion_macros::user_doc;
 use std::any::Any;
-use std::sync::{Arc, OnceLock};
+use std::sync::Arc;
 
 make_udf_expr_and_func!(
     MapKeysFunc,
@@ -37,11 +38,35 @@ make_udf_expr_and_func!(
     map_keys_udf
 );
 
+#[user_doc(
+    doc_section(label = "Map Functions"),
+    description = "Returns a list of all keys in the map.",
+    syntax_example = "map_keys(map)",
+    sql_example = r#"```sql
+SELECT map_keys(MAP {'a': 1, 'b': NULL, 'c': 3});
+----
+[a, b, c]
+
+SELECT map_keys(map([100, 5], [42, 43]));
+----
+[100, 5]
+```"#,
+    argument(
+        name = "map",
+        description = "Map expression. Can be a constant, column, or function, and any combination of map operators."
+    )
+)]
 #[derive(Debug)]
-pub(crate) struct MapKeysFunc {
+pub struct MapKeysFunc {
     signature: Signature,
 }
 
+impl Default for MapKeysFunc {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl MapKeysFunc {
     pub fn new() -> Self {
         Self {
@@ -67,10 +92,7 @@ impl ScalarUDFImpl for MapKeysFunc {
     }
 
     fn return_type(&self, arg_types: &[DataType]) -> Result<DataType> {
-        if arg_types.len() != 1 {
-            return exec_err!("map_keys expects single argument");
-        }
-        let map_type = &arg_types[0];
+        let [map_type] = take_function_args(self.name(), arg_types)?;
         let map_fields = get_map_entry_field(map_type)?;
         Ok(DataType::List(Arc::new(Field::new_list_field(
             map_fields.first().unwrap().data_type().clone(),
@@ -78,53 +100,23 @@ impl ScalarUDFImpl for MapKeysFunc {
         ))))
     }
 
-    fn invoke_batch(
+    fn invoke_with_args(
         &self,
-        args: &[ColumnarValue],
-        _number_rows: usize,
+        args: datafusion_expr::ScalarFunctionArgs,
     ) -> Result<ColumnarValue> {
-        make_scalar_function(map_keys_inner)(args)
+        make_scalar_function(map_keys_inner)(&args.args)
     }
 
     fn documentation(&self) -> Option<&Documentation> {
-        Some(get_map_keys_doc())
+        self.doc()
     }
 }
 
-static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
-
-fn get_map_keys_doc() -> &'static Documentation {
-    DOCUMENTATION.get_or_init(|| {
-        Documentation::builder(
-            DOC_SECTION_MAP,
-                "Returns a list of all keys in the map.",
-            "map_keys(map)")
-            .with_sql_example(
-                r#"```sql
-SELECT map_keys(MAP {'a': 1, 'b': NULL, 'c': 3});
-----
-[a, b, c]
-
-SELECT map_keys(map([100, 5], [42, 43]));
-----
-[100, 5]
-```"#,
-            )
-            .with_argument(
-                "map",
-                "Map expression. Can be a constant, column, or function, and any combination of map operators."
-            )
-            .build()
-    })
-}
-
 fn map_keys_inner(args: &[ArrayRef]) -> Result<ArrayRef> {
-    if args.len() != 1 {
-        return exec_err!("map_keys expects single argument");
-    }
+    let [map_arg] = take_function_args("map_keys", args)?;
 
-    let map_array = match args[0].data_type() {
-        DataType::Map(_, _) => as_map_array(&args[0])?,
+    let map_array = match map_arg.data_type() {
+        DataType::Map(_, _) => as_map_array(&map_arg)?,
         _ => return exec_err!("Argument for map_keys should be a map"),
     };
 
@@ -132,6 +124,6 @@ fn map_keys_inner(args: &[ArrayRef]) -> Result<ArrayRef> {
         Arc::new(Field::new_list_field(map_array.key_type().clone(), false)),
         map_array.offsets().clone(),
         Arc::clone(map_array.keys()),
-        None,
+        map_array.nulls().cloned(),
     )))
 }
