@@ -19,14 +19,13 @@
 
 use std::sync::Arc;
 
-use arrow::{array::ArrayRef, datatypes::DataType};
+use arrow::datatypes::{DataType, Field, Fields};
 
-use arrow_array::{
-    Array, BooleanArray, GenericListArray, ListArray, OffsetSizeTrait, Scalar,
+use arrow::array::{
+    Array, ArrayRef, BooleanArray, GenericListArray, ListArray, OffsetSizeTrait, Scalar,
     UInt32Array,
 };
-use arrow_buffer::OffsetBuffer;
-use arrow_schema::{Field, Fields};
+use arrow::buffer::OffsetBuffer;
 use datafusion_common::cast::{as_large_list_array, as_list_array};
 use datafusion_common::{
     exec_err, internal_datafusion_err, internal_err, plan_err, Result, ScalarValue,
