@@ -197,7 +197,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         let regex = Arc::new(regex(&mut rng)) as ArrayRef;
         let flags = Arc::new(flags(&mut rng)) as ArrayRef;
         let replacement =
-            Arc::new(StringArray::from_iter_values(iter::repeat("XX").take(1000)))
+            Arc::new(StringArray::from_iter_values(iter::repeat_n("XX", 1000)))
                 as ArrayRef;
 
         b.iter(|| {
@@ -219,9 +219,9 @@ fn criterion_benchmark(c: &mut Criterion) {
         let regex = cast(&regex(&mut rng), &DataType::Utf8View).unwrap();
         // flags are not allowed to be utf8view according to the function
         let flags = Arc::new(flags(&mut rng)) as ArrayRef;
-        let replacement = Arc::new(StringViewArray::from_iter_values(
-            iter::repeat("XX").take(1000),
-        ));
+        let replacement = Arc::new(StringViewArray::from_iter_values(iter::repeat_n(
+            "XX", 1000,
+        )));
 
         b.iter(|| {
             black_box(
