@@ -17,12 +17,13 @@
 
 extern crate criterion;
 
+use arrow::datatypes::DataType;
 use arrow::{
     datatypes::{Float32Type, Float64Type},
     util::bench_util::create_primitive_array,
 };
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
-use datafusion_expr::ColumnarValue;
+use datafusion_expr::{ColumnarValue, ScalarFunctionArgs};
 use datafusion_functions::math::signum;
 use std::sync::Arc;
 
@@ -34,8 +35,15 @@ fn criterion_benchmark(c: &mut Criterion) {
         let f32_args = vec![ColumnarValue::Array(f32_array)];
         c.bench_function(&format!("signum f32 array: {}", size), |b| {
             b.iter(|| {
-                // TODO use invoke_with_args
-                black_box(signum.invoke_batch(&f32_args, batch_len).unwrap())
+                black_box(
+                    signum
+                        .invoke_with_args(ScalarFunctionArgs {
+                            args: f32_args.clone(),
+                            number_rows: batch_len,
+                            return_type: &DataType::Float32,
+                        })
+                        .unwrap(),
+                )
             })
         });
         let f64_array = Arc::new(create_primitive_array::<Float64Type>(size, 0.2));
@@ -44,8 +52,15 @@ fn criterion_benchmark(c: &mut Criterion) {
         let f64_args = vec![ColumnarValue::Array(f64_array)];
         c.bench_function(&format!("signum f64 array: {}", size), |b| {
             b.iter(|| {
-                // TODO use invoke_with_args
-                black_box(signum.invoke_batch(&f64_args, batch_len).unwrap())
+                black_box(
+                    signum
+                        .invoke_with_args(ScalarFunctionArgs {
+                            args: f64_args.clone(),
+                            number_rows: batch_len,
+                            return_type: &DataType::Float64,
+                        })
+                        .unwrap(),
+                )
             })
         });
     }
