@@ -19,15 +19,33 @@ use arrow::array::{new_null_array, BooleanArray};
 use arrow::compute::kernels::zip::zip;
 use arrow::compute::{and, is_not_null, is_null};
 use arrow::datatypes::DataType;
-use datafusion_common::{exec_err, ExprSchema, Result};
+use datafusion_common::{exec_err, internal_err, Result};
 use datafusion_expr::binary::try_type_union_resolution;
-use datafusion_expr::scalar_doc_sections::DOC_SECTION_CONDITIONAL;
-use datafusion_expr::{ColumnarValue, Documentation, Expr, ExprSchemable};
+use datafusion_expr::{
+    ColumnarValue, Documentation, ReturnInfo, ReturnTypeArgs, ScalarFunctionArgs,
+};
 use datafusion_expr::{ScalarUDFImpl, Signature, Volatility};
+use datafusion_macros::user_doc;
 use itertools::Itertools;
 use std::any::Any;
-use std::sync::OnceLock;
 
+#[user_doc(
+    doc_section(label = "Conditional Functions"),
+    description = "Returns the first of its arguments that is not _null_. Returns _null_ if all arguments are _null_. This function is often used to substitute a default value for _null_ values.",
+    syntax_example = "coalesce(expression1[, ..., expression_n])",
+    sql_example = r#"```sql
+> select coalesce(null, null, 'datafusion');
++----------------------------------------+
+| coalesce(NULL,NULL,Utf8("datafusion")) |
++----------------------------------------+
+| datafusion                             |
++----------------------------------------+
+```"#,
+    argument(
+        name = "expression1, expression_n",
+        description = "Expression to use if previous expressions are _null_. Can be a constant, column, or function, and any combination of arithmetic operators. Pass as many expression arguments as necessary."
+    )
+)]
 #[derive(Debug)]
 pub struct CoalesceFunc {
     signature: Signature,
@@ -60,25 +78,25 @@ impl ScalarUDFImpl for CoalesceFunc {
         &self.signature
     }
 
-    fn return_type(&self, arg_types: &[DataType]) -> Result<DataType> {
-        Ok(arg_types
+    fn return_type(&self, _arg_types: &[DataType]) -> Result<DataType> {
+        internal_err!("return_type_from_args should be called instead")
+    }
+
+    fn return_type_from_args(&self, args: ReturnTypeArgs) -> Result<ReturnInfo> {
+        // If any the arguments in coalesce is non-null, the result is non-null
+        let nullable = args.nullables.iter().all(|&nullable| nullable);
+        let return_type = args
+            .arg_types
             .iter()
             .find_or_first(|d| !d.is_null())
             .unwrap()
-            .clone())
-    }
-
-    // If any the arguments in coalesce is non-null, the result is non-null
-    fn is_nullable(&self, args: &[Expr], schema: &dyn ExprSchema) -> bool {
-        args.iter().all(|e| e.nullable(schema).ok().unwrap_or(true))
+            .clone();
+        Ok(ReturnInfo::new(return_type, nullable))
     }
 
     /// coalesce evaluates to the first value which is not NULL
-    fn invoke_batch(
-        &self,
-        args: &[ColumnarValue],
-        _number_rows: usize,
-    ) -> Result<ColumnarValue> {
+    fn invoke_with_args(&self, args: ScalarFunctionArgs) -> Result<ColumnarValue> {
+        let args = args.args;
         // do not accept 0 arguments.
         if args.is_empty() {
             return exec_err!(
@@ -146,67 +164,6 @@ impl ScalarUDFImpl for CoalesceFunc {
     }
 
     fn documentation(&self) -> Option<&Documentation> {
-        Some(get_coalesce_doc())
-    }
-}
-
-static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
-
-fn get_coalesce_doc() -> &'static Documentation {
-    DOCUMENTATION.get_or_init(|| {
-        Documentation::builder(
-            DOC_SECTION_CONDITIONAL,
-            "Returns the first of its arguments that is not _null_. Returns _null_ if all arguments are _null_. This function is often used to substitute a default value for _null_ values.",
-            "coalesce(expression1[, ..., expression_n])")
-            .with_sql_example(r#"```sql
-> select coalesce(null, null, 'datafusion');
-+----------------------------------------+
-| coalesce(NULL,NULL,Utf8("datafusion")) |
-+----------------------------------------+
-| datafusion                             |
-+----------------------------------------+
-```"#,
-            )
-            .with_argument(
-                "expression1, expression_n",
-                "Expression to use if previous expressions are _null_. Can be a constant, column, or function, and any combination of arithmetic operators. Pass as many expression arguments as necessary."
-            )
-            .build()
-    })
-}
-
-#[cfg(test)]
-mod test {
-    use arrow::datatypes::DataType;
-
-    use datafusion_expr::ScalarUDFImpl;
-
-    use crate::core;
-
-    #[test]
-    fn test_coalesce_return_types() {
-        let coalesce = core::coalesce::CoalesceFunc::new();
-        let return_type = coalesce
-            .return_type(&[DataType::Date32, DataType::Date32])
-            .unwrap();
-        assert_eq!(return_type, DataType::Date32);
-    }
-
-    #[test]
-    fn test_coalesce_return_types_with_nulls_first() {
-        let coalesce = core::coalesce::CoalesceFunc::new();
-        let return_type = coalesce
-            .return_type(&[DataType::Null, DataType::Date32])
-            .unwrap();
-        assert_eq!(return_type, DataType::Date32);
-    }
-
-    #[test]
-    fn test_coalesce_return_types_with_nulls_last() {
-        let coalesce = core::coalesce::CoalesceFunc::new();
-        let return_type = coalesce
-            .return_type(&[DataType::Int64, DataType::Null])
-            .unwrap();
-        assert_eq!(return_type, DataType::Int64);
+        self.doc()
     }
 }
