@@ -16,7 +16,7 @@
 // under the License.
 
 use std::any::Any;
-use std::sync::{Arc, OnceLock};
+use std::sync::Arc;
 
 use arrow::array::temporal_conversions::NANOSECONDS;
 use arrow::array::types::{
@@ -37,10 +37,64 @@ use datafusion_expr::TypeSignature::Exact;
 use datafusion_expr::{
     ColumnarValue, Documentation, ScalarUDFImpl, Signature, Volatility, TIMEZONE_WILDCARD,
 };
+use datafusion_macros::user_doc;
 
 use chrono::{DateTime, Datelike, Duration, Months, TimeDelta, Utc};
-use datafusion_expr::scalar_doc_sections::DOC_SECTION_DATETIME;
 
+#[user_doc(
+    doc_section(label = "Time and Date Functions"),
+    description = r#"
+Calculates time intervals and returns the start of the interval nearest to the specified timestamp. Use `date_bin` to downsample time series data by grouping rows into time-based "bins" or "windows" and applying an aggregate or selector function to each window.
+
+For example, if you "bin" or "window" data into 15 minute intervals, an input timestamp of `2023-01-01T18:18:18Z` will be updated to the start time of the 15 minute bin it is in: `2023-01-01T18:15:00Z`.
+"#,
+    syntax_example = "date_bin(interval, expression, origin-timestamp)",
+    sql_example = r#"```sql
+-- Bin the timestamp into 1 day intervals
+> SELECT date_bin(interval '1 day', time) as bin
+FROM VALUES ('2023-01-01T18:18:18Z'), ('2023-01-03T19:00:03Z')  t(time);
++---------------------+
+| bin                 |
++---------------------+
+| 2023-01-01T00:00:00 |
+| 2023-01-03T00:00:00 |
++---------------------+
+2 row(s) fetched.
+
+-- Bin the timestamp into 1 day intervals starting at 3AM on  2023-01-01
+> SELECT date_bin(interval '1 day', time,  '2023-01-01T03:00:00') as bin
+FROM VALUES ('2023-01-01T18:18:18Z'), ('2023-01-03T19:00:03Z')  t(time);
++---------------------+
+| bin                 |
++---------------------+
+| 2023-01-01T03:00:00 |
+| 2023-01-03T03:00:00 |
++---------------------+
+2 row(s) fetched.
+```"#,
+    argument(name = "interval", description = "Bin interval."),
+    argument(
+        name = "expression",
+        description = "Time expression to operate on. Can be a constant, column, or function."
+    ),
+    argument(
+        name = "origin-timestamp",
+        description = r#"Optional. Starting point used to determine bin boundaries. If not specified defaults 1970-01-01T00:00:00Z (the UNIX epoch in UTC). The following intervals are supported:
+
+    - nanoseconds
+    - microseconds
+    - milliseconds
+    - seconds
+    - minutes
+    - hours
+    - days
+    - weeks
+    - months
+    - years
+    - century
+"#
+    )
+)]
 #[derive(Debug)]
 pub struct DateBinFunc {
     signature: Signature,
@@ -133,11 +187,11 @@ impl ScalarUDFImpl for DateBinFunc {
         }
     }
 
-    fn invoke_batch(
+    fn invoke_with_args(
         &self,
-        args: &[ColumnarValue],
-        _number_rows: usize,
+        args: datafusion_expr::ScalarFunctionArgs,
     ) -> Result<ColumnarValue> {
+        let args = &args.args;
         if args.len() == 2 {
             // Default to unix EPOCH
             let origin = ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(
@@ -169,68 +223,10 @@ impl ScalarUDFImpl for DateBinFunc {
         }
     }
     fn documentation(&self) -> Option<&Documentation> {
-        Some(get_date_bin_doc())
+        self.doc()
     }
 }
 
-static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
-
-fn get_date_bin_doc() -> &'static Documentation {
-    DOCUMENTATION.get_or_init(|| {
-        Documentation::builder(
-            DOC_SECTION_DATETIME,
-            r#"
-Calculates time intervals and returns the start of the interval nearest to the specified timestamp. Use `date_bin` to downsample time series data by grouping rows into time-based "bins" or "windows" and applying an aggregate or selector function to each window.
-
-For example, if you "bin" or "window" data into 15 minute intervals, an input timestamp of `2023-01-01T18:18:18Z` will be updated to the start time of the 15 minute bin it is in: `2023-01-01T18:15:00Z`.
-"#,
-            "date_bin(interval, expression, origin-timestamp)")
-            .with_sql_example(r#"```sql
--- Bin the timestamp into 1 day intervals
-> SELECT date_bin(interval '1 day', time) as bin
-FROM VALUES ('2023-01-01T18:18:18Z'), ('2023-01-03T19:00:03Z')  t(time);
-+---------------------+
-| bin                 |
-+---------------------+
-| 2023-01-01T00:00:00 |
-| 2023-01-03T00:00:00 |
-+---------------------+
-2 row(s) fetched.
-
--- Bin the timestamp into 1 day intervals starting at 3AM on  2023-01-01
-> SELECT date_bin(interval '1 day', time,  '2023-01-01T03:00:00') as bin
-FROM VALUES ('2023-01-01T18:18:18Z'), ('2023-01-03T19:00:03Z')  t(time);
-+---------------------+
-| bin                 |
-+---------------------+
-| 2023-01-01T03:00:00 |
-| 2023-01-03T03:00:00 |
-+---------------------+
-2 row(s) fetched.
-```
-"#)
-            .with_argument("interval", "Bin interval.")
-            .with_argument("expression", "Time expression to operate on. Can be a constant, column, or function.")
-            .with_argument("origin-timestamp", "Optional. Starting point used to determine bin boundaries. If not specified defaults 1970-01-01T00:00:00Z (the UNIX epoch in UTC).
-
-The following intervals are supported:
-
-- nanoseconds
-- microseconds
-- milliseconds
-- seconds
-- minutes
-- hours
-- days
-- weeks
-- months
-- years
-- century
-")
-            .build()
-    })
-}
-
 enum Interval {
     Nanoseconds(i64),
     Months(i64),
@@ -518,10 +514,9 @@ mod tests {
     use chrono::TimeDelta;
 
     #[test]
-    #[allow(deprecated)] // TODO migrate UDF invoke from invoke_batch
     fn test_date_bin() {
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        let mut args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::IntervalDayTime(Some(
                     IntervalDayTime {
                         days: 0,
@@ -531,14 +526,16 @@ mod tests {
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert!(res.is_ok());
 
         let timestamps = Arc::new((1..6).map(Some).collect::<TimestampNanosecondArray>());
         let batch_len = timestamps.len();
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::IntervalDayTime(Some(
                     IntervalDayTime {
                         days: 0,
@@ -548,12 +545,14 @@ mod tests {
                 ColumnarValue::Array(timestamps),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            batch_len,
-        );
+            number_rows: batch_len,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert!(res.is_ok());
 
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::IntervalDayTime(Some(
                     IntervalDayTime {
                         days: 0,
@@ -562,13 +561,15 @@ mod tests {
                 ))),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert!(res.is_ok());
 
         // stride supports month-day-nano
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::IntervalMonthDayNano(Some(
                     IntervalMonthDayNano {
                         months: 0,
@@ -579,8 +580,10 @@ mod tests {
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert!(res.is_ok());
 
         //
@@ -588,37 +591,42 @@ mod tests {
         //
 
         // invalid number of arguments
-        let res = DateBinFunc::new().invoke_batch(
-            &[ColumnarValue::Scalar(ScalarValue::IntervalDayTime(Some(
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![ColumnarValue::Scalar(ScalarValue::IntervalDayTime(Some(
                 IntervalDayTime {
                     days: 0,
                     milliseconds: 1,
                 },
             )))],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "Execution error: DATE_BIN expected two or three arguments"
         );
 
         // stride: invalid type
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::IntervalYearMonth(Some(1))),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "Execution error: DATE_BIN expects stride argument to be an INTERVAL but got Interval(YearMonth)"
         );
 
         // stride: invalid value
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::IntervalDayTime(Some(
                     IntervalDayTime {
                         days: 0,
@@ -628,60 +636,69 @@ mod tests {
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "Execution error: DATE_BIN stride must be non-zero"
         );
 
         // stride: overflow of day-time interval
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::IntervalDayTime(Some(
                     IntervalDayTime::MAX,
                 ))),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "Execution error: DATE_BIN stride argument is too large"
         );
 
         // stride: overflow of month-day-nano interval
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::new_interval_mdn(0, i32::MAX, 1)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "Execution error: DATE_BIN stride argument is too large"
         );
 
         // stride: month intervals
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::new_interval_mdn(1, 1, 1)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "This feature is not implemented: DATE_BIN stride does not support combination of month, day and nanosecond intervals"
         );
 
         // origin: invalid type
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::IntervalDayTime(Some(
                     IntervalDayTime {
                         days: 0,
@@ -691,15 +708,17 @@ mod tests {
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
                 ColumnarValue::Scalar(ScalarValue::TimestampMicrosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "Execution error: DATE_BIN expects origin argument to be a TIMESTAMP with nanosecond precision but got Timestamp(Microsecond, None)"
         );
 
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::IntervalDayTime(Some(
                     IntervalDayTime {
                         days: 0,
@@ -709,8 +728,10 @@ mod tests {
                 ColumnarValue::Scalar(ScalarValue::TimestampMicrosecond(Some(1), None)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert!(res.is_ok());
 
         // unsupported array type for stride
@@ -724,14 +745,16 @@ mod tests {
                 })
                 .collect::<IntervalDayTimeArray>(),
         );
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Array(intervals),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "This feature is not implemented: DATE_BIN only supports literal values for the stride argument, not arrays"
@@ -740,8 +763,8 @@ mod tests {
         // unsupported array type for origin
         let timestamps = Arc::new((1..6).map(Some).collect::<TimestampNanosecondArray>());
         let batch_len = timestamps.len();
-        let res = DateBinFunc::new().invoke_batch(
-            &[
+        args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::IntervalDayTime(Some(
                     IntervalDayTime {
                         days: 0,
@@ -751,8 +774,10 @@ mod tests {
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
                 ColumnarValue::Array(timestamps),
             ],
-            batch_len,
-        );
+            number_rows: batch_len,
+            return_type: &DataType::Timestamp(TimeUnit::Nanosecond, None),
+        };
+        let res = DateBinFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "This feature is not implemented: DATE_BIN only supports literal values for the origin argument, not arrays"
@@ -868,20 +893,22 @@ mod tests {
                     .collect::<TimestampNanosecondArray>()
                     .with_timezone_opt(tz_opt.clone());
                 let batch_len = input.len();
-                #[allow(deprecated)] // TODO migrate UDF invoke to invoke_batch
-                let result = DateBinFunc::new()
-                    .invoke_batch(
-                        &[
-                            ColumnarValue::Scalar(ScalarValue::new_interval_dt(1, 0)),
-                            ColumnarValue::Array(Arc::new(input)),
-                            ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(
-                                Some(string_to_timestamp_nanos(origin).unwrap()),
-                                tz_opt.clone(),
-                            )),
-                        ],
-                        batch_len,
-                    )
-                    .unwrap();
+                let args = datafusion_expr::ScalarFunctionArgs {
+                    args: vec![
+                        ColumnarValue::Scalar(ScalarValue::new_interval_dt(1, 0)),
+                        ColumnarValue::Array(Arc::new(input)),
+                        ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(
+                            Some(string_to_timestamp_nanos(origin).unwrap()),
+                            tz_opt.clone(),
+                        )),
+                    ],
+                    number_rows: batch_len,
+                    return_type: &DataType::Timestamp(
+                        TimeUnit::Nanosecond,
+                        tz_opt.clone(),
+                    ),
+                };
+                let result = DateBinFunc::new().invoke_with_args(args).unwrap();
                 if let ColumnarValue::Array(result) = result {
                     assert_eq!(
                         result.data_type(),
