@@ -16,7 +16,7 @@
 // under the License.
 
 use std::any::Any;
-use std::sync::{Arc, OnceLock};
+use std::sync::Arc;
 
 use arrow::array::builder::PrimitiveBuilder;
 use arrow::array::cast::AsArray;
@@ -26,12 +26,46 @@ use arrow::datatypes::DataType;
 use arrow::datatypes::DataType::{Date32, Int32, Int64, UInt32, UInt64, Utf8, Utf8View};
 use chrono::prelude::*;
 
-use datafusion_common::{exec_err, Result, ScalarValue};
-use datafusion_expr::scalar_doc_sections::DOC_SECTION_DATETIME;
+use datafusion_common::{exec_err, utils::take_function_args, Result, ScalarValue};
 use datafusion_expr::{
     ColumnarValue, Documentation, ScalarUDFImpl, Signature, Volatility,
 };
+use datafusion_macros::user_doc;
 
+#[user_doc(
+    doc_section(label = "Time and Date Functions"),
+    description = "Make a date from year/month/day component parts.",
+    syntax_example = "make_date(year, month, day)",
+    sql_example = r#"```sql
+> select make_date(2023, 1, 31);
++-------------------------------------------+
+| make_date(Int64(2023),Int64(1),Int64(31)) |
++-------------------------------------------+
+| 2023-01-31                                |
++-------------------------------------------+
+> select make_date('2023', '01', '31');
++-----------------------------------------------+
+| make_date(Utf8("2023"),Utf8("01"),Utf8("31")) |
++-----------------------------------------------+
+| 2023-01-31                                    |
++-----------------------------------------------+
+```
+
+Additional examples can be found [here](https://github.com/apache/datafusion/blob/main/datafusion-examples/examples/make_date.rs)
+"#,
+    argument(
+        name = "year",
+        description = "Year to use when making the date. Can be a constant, column or function, and any combination of arithmetic operators."
+    ),
+    argument(
+        name = "month",
+        description = "Month to use when making the date. Can be a constant, column or function, and any combination of arithmetic operators."
+    ),
+    argument(
+        name = "day",
+        description = "Day to use when making the date. Can be a constant, column or function, and any combination of arithmetic operators."
+    )
+)]
 #[derive(Debug)]
 pub struct MakeDateFunc {
     signature: Signature,
@@ -72,20 +106,13 @@ impl ScalarUDFImpl for MakeDateFunc {
         Ok(Date32)
     }
 
-    fn invoke_batch(
+    fn invoke_with_args(
         &self,
-        args: &[ColumnarValue],
-        _number_rows: usize,
+        args: datafusion_expr::ScalarFunctionArgs,
     ) -> Result<ColumnarValue> {
-        if args.len() != 3 {
-            return exec_err!(
-                "make_date function requires 3 arguments, got {}",
-                args.len()
-            );
-        }
-
         // first, identify if any of the arguments is an Array. If yes, store its `len`,
         // as any scalar will need to be converted to an array of len `len`.
+        let args = args.args;
         let len = args
             .iter()
             .fold(Option::<usize>::None, |acc, arg| match arg {
@@ -93,9 +120,11 @@ impl ScalarUDFImpl for MakeDateFunc {
                 ColumnarValue::Array(a) => Some(a.len()),
             });
 
-        let years = args[0].cast_to(&Int32, None)?;
-        let months = args[1].cast_to(&Int32, None)?;
-        let days = args[2].cast_to(&Int32, None)?;
+        let [years, months, days] = take_function_args(self.name(), args)?;
+
+        let years = years.cast_to(&Int32, None)?;
+        let months = months.cast_to(&Int32, None)?;
+        let days = days.cast_to(&Int32, None)?;
 
         let scalar_value_fn = |col: &ColumnarValue| -> Result<i32> {
             let ColumnarValue::Scalar(s) = col else {
@@ -156,47 +185,10 @@ impl ScalarUDFImpl for MakeDateFunc {
         Ok(value)
     }
     fn documentation(&self) -> Option<&Documentation> {
-        Some(get_make_date_doc())
+        self.doc()
     }
 }
 
-static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
-
-fn get_make_date_doc() -> &'static Documentation {
-    DOCUMENTATION.get_or_init(|| {
-        Documentation::builder(
-            DOC_SECTION_DATETIME,
-            "Make a date from year/month/day component parts.",
-            "make_date(year, month, day)")
-            .with_argument(
-                "year",
-                " Year to use when making the date. Can be a constant, column or function, and any combination of arithmetic operators.", )
-            .with_argument(
-                "month",
-                "Month to use when making the date. Can be a constant, column or function, and any combination of arithmetic operators.",
-            )
-            .with_argument("day", "Day to use when making the date. Can be a constant, column or function, and any combination of arithmetic operators.")
-            .with_sql_example(r#"```sql
-> select make_date(2023, 1, 31);
-+-------------------------------------------+
-| make_date(Int64(2023),Int64(1),Int64(31)) |
-+-------------------------------------------+
-| 2023-01-31                                |
-+-------------------------------------------+
-> select make_date('2023', '01', '31');
-+-----------------------------------------------+
-| make_date(Utf8("2023"),Utf8("01"),Utf8("31")) |
-+-----------------------------------------------+
-| 2023-01-31                                    |
-+-----------------------------------------------+
-```
-
-Additional examples can be found [here](https://github.com/apache/datafusion/blob/main/datafusion-examples/examples/make_date.rs)
-"#)
-            .build()
-    })
-}
-
 /// Converts the year/month/day fields to an `i32` representing the days from
 /// the unix epoch and invokes `date_consumer_fn` with the value
 fn make_date_inner<F: FnMut(i32)>(
@@ -231,22 +223,24 @@ fn make_date_inner<F: FnMut(i32)>(
 mod tests {
     use crate::datetime::make_date::MakeDateFunc;
     use arrow::array::{Array, Date32Array, Int32Array, Int64Array, UInt32Array};
+    use arrow::datatypes::DataType;
     use datafusion_common::ScalarValue;
     use datafusion_expr::{ColumnarValue, ScalarUDFImpl};
     use std::sync::Arc;
 
     #[test]
     fn test_make_date() {
-        #[allow(deprecated)] // TODO migrate UDF to invoke from invoke_batch
+        let args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
+                ColumnarValue::Scalar(ScalarValue::Int32(Some(2024))),
+                ColumnarValue::Scalar(ScalarValue::Int64(Some(1))),
+                ColumnarValue::Scalar(ScalarValue::UInt32(Some(14))),
+            ],
+            number_rows: 1,
+            return_type: &DataType::Date32,
+        };
         let res = MakeDateFunc::new()
-            .invoke_batch(
-                &[
-                    ColumnarValue::Scalar(ScalarValue::Int32(Some(2024))),
-                    ColumnarValue::Scalar(ScalarValue::Int64(Some(1))),
-                    ColumnarValue::Scalar(ScalarValue::UInt32(Some(14))),
-                ],
-                1,
-            )
+            .invoke_with_args(args)
             .expect("that make_date parsed values without error");
 
         if let ColumnarValue::Scalar(ScalarValue::Date32(date)) = res {
@@ -255,16 +249,17 @@ mod tests {
             panic!("Expected a scalar value")
         }
 
-        #[allow(deprecated)] // TODO migrate UDF to invoke from invoke_batch
+        let args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
+                ColumnarValue::Scalar(ScalarValue::Int64(Some(2024))),
+                ColumnarValue::Scalar(ScalarValue::UInt64(Some(1))),
+                ColumnarValue::Scalar(ScalarValue::UInt32(Some(14))),
+            ],
+            number_rows: 1,
+            return_type: &DataType::Date32,
+        };
         let res = MakeDateFunc::new()
-            .invoke_batch(
-                &[
-                    ColumnarValue::Scalar(ScalarValue::Int64(Some(2024))),
-                    ColumnarValue::Scalar(ScalarValue::UInt64(Some(1))),
-                    ColumnarValue::Scalar(ScalarValue::UInt32(Some(14))),
-                ],
-                1,
-            )
+            .invoke_with_args(args)
             .expect("that make_date parsed values without error");
 
         if let ColumnarValue::Scalar(ScalarValue::Date32(date)) = res {
@@ -273,16 +268,17 @@ mod tests {
             panic!("Expected a scalar value")
         }
 
-        #[allow(deprecated)] // TODO migrate UDF to invoke from invoke_batch
+        let args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
+                ColumnarValue::Scalar(ScalarValue::Utf8(Some("2024".to_string()))),
+                ColumnarValue::Scalar(ScalarValue::LargeUtf8(Some("1".to_string()))),
+                ColumnarValue::Scalar(ScalarValue::Utf8(Some("14".to_string()))),
+            ],
+            number_rows: 1,
+            return_type: &DataType::Date32,
+        };
         let res = MakeDateFunc::new()
-            .invoke_batch(
-                &[
-                    ColumnarValue::Scalar(ScalarValue::Utf8(Some("2024".to_string()))),
-                    ColumnarValue::Scalar(ScalarValue::LargeUtf8(Some("1".to_string()))),
-                    ColumnarValue::Scalar(ScalarValue::Utf8(Some("14".to_string()))),
-                ],
-                1,
-            )
+            .invoke_with_args(args)
             .expect("that make_date parsed values without error");
 
         if let ColumnarValue::Scalar(ScalarValue::Date32(date)) = res {
@@ -295,16 +291,17 @@ mod tests {
         let months = Arc::new((1..5).map(Some).collect::<Int32Array>());
         let days = Arc::new((11..15).map(Some).collect::<UInt32Array>());
         let batch_len = years.len();
-        #[allow(deprecated)] // TODO migrate UDF to invoke from invoke_batch
+        let args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
+                ColumnarValue::Array(years),
+                ColumnarValue::Array(months),
+                ColumnarValue::Array(days),
+            ],
+            number_rows: batch_len,
+            return_type: &DataType::Date32,
+        };
         let res = MakeDateFunc::new()
-            .invoke_batch(
-                &[
-                    ColumnarValue::Array(years),
-                    ColumnarValue::Array(months),
-                    ColumnarValue::Array(days),
-                ],
-                batch_len,
-            )
+            .invoke_with_args(args)
             .expect("that make_date parsed values without error");
 
         if let ColumnarValue::Array(array) = res {
@@ -324,54 +321,60 @@ mod tests {
         //
 
         // invalid number of arguments
-        #[allow(deprecated)] // TODO migrate UDF to invoke from invoke_batch
-        let res = MakeDateFunc::new()
-            .invoke_batch(&[ColumnarValue::Scalar(ScalarValue::Int32(Some(1)))], 1);
+        let args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![ColumnarValue::Scalar(ScalarValue::Int32(Some(1)))],
+            number_rows: 1,
+            return_type: &DataType::Date32,
+        };
+        let res = MakeDateFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "Execution error: make_date function requires 3 arguments, got 1"
         );
 
         // invalid type
-        #[allow(deprecated)] // TODO migrate UDF to invoke from invoke_batch
-        let res = MakeDateFunc::new().invoke_batch(
-            &[
+        let args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::IntervalYearMonth(Some(1))),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
                 ColumnarValue::Scalar(ScalarValue::TimestampNanosecond(Some(1), None)),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Date32,
+        };
+        let res = MakeDateFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "Arrow error: Cast error: Casting from Interval(YearMonth) to Int32 not supported"
         );
 
         // overflow of month
-        #[allow(deprecated)] // TODO migrate UDF to invoke from invoke_batch
-        let res = MakeDateFunc::new().invoke_batch(
-            &[
+        let args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::Int32(Some(2023))),
                 ColumnarValue::Scalar(ScalarValue::UInt64(Some(u64::MAX))),
                 ColumnarValue::Scalar(ScalarValue::Int32(Some(22))),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Date32,
+        };
+        let res = MakeDateFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "Arrow error: Cast error: Can't cast value 18446744073709551615 to type Int32"
         );
 
         // overflow of day
-        #[allow(deprecated)] // TODO migrate UDF to invoke from invoke_batch
-        let res = MakeDateFunc::new().invoke_batch(
-            &[
+        let args = datafusion_expr::ScalarFunctionArgs {
+            args: vec![
                 ColumnarValue::Scalar(ScalarValue::Int32(Some(2023))),
                 ColumnarValue::Scalar(ScalarValue::Int32(Some(22))),
                 ColumnarValue::Scalar(ScalarValue::UInt32(Some(u32::MAX))),
             ],
-            1,
-        );
+            number_rows: 1,
+            return_type: &DataType::Date32,
+        };
+        let res = MakeDateFunc::new().invoke_with_args(args);
         assert_eq!(
             res.err().unwrap().strip_backtrace(),
             "Arrow error: Cast error: Can't cast value 4294967295 to type Int32"
