@@ -15,7 +15,14 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#![doc(
+    html_logo_url = "https://raw.githubusercontent.com/apache/datafusion/19fe44cf2f30cbdd63d4a4f52c74055163c6cc38/docs/logos/standalone_logo/logo_original.svg",
+    html_favicon_url = "https://raw.githubusercontent.com/apache/datafusion/19fe44cf2f30cbdd63d4a4f52c74055163c6cc38/docs/logos/standalone_logo/logo_original.svg"
+)]
+#![cfg_attr(docsrs, feature(doc_auto_cfg))]
+
 extern crate proc_macro;
+use datafusion_expr::scalar_doc_sections::doc_sections_const;
 use proc_macro::TokenStream;
 use quote::quote;
 use syn::{parse_macro_input, DeriveInput, LitStr};
@@ -24,11 +31,12 @@ use syn::{parse_macro_input, DeriveInput, LitStr};
 /// from it by constructing a `DocumentBuilder()` automatically. The `Documentation` can be
 /// retrieved from the `documentation()` method
 /// declared on `AggregateUDF`, `WindowUDFImpl`, `ScalarUDFImpl` traits.
-///
+/// For `doc_section`, this macro will try to find corresponding predefined `DocSection` by label field
+/// Predefined `DocSection` can be found in datafusion/expr/src/udf.rs
 /// Example:
 /// ```ignore
 /// #[user_doc(
-///     doc_section(include = "true", label = "Time and Date Functions"),
+///     doc_section(label = "Time and Date Functions"),
 ///     description = r"Converts a value to a date (`YYYY-MM-DD`).",
 ///     syntax_example = "to_date('2017-05-31', '%Y-%m-%d')",
 ///     sql_example = r#"```sql
@@ -97,13 +105,11 @@ use syn::{parse_macro_input, DeriveInput, LitStr};
 /// ```
 #[proc_macro_attribute]
 pub fn user_doc(args: TokenStream, input: TokenStream) -> TokenStream {
-    let mut doc_section_include: Option<LitStr> = None;
     let mut doc_section_lbl: Option<LitStr> = None;
-    let mut doc_section_desc: Option<LitStr> = None;
 
     let mut description: Option<LitStr> = None;
     let mut syntax_example: Option<LitStr> = None;
-    let mut alt_syntax_example: Option<LitStr> = None;
+    let mut alt_syntax_example: Vec<Option<LitStr>> = vec![];
     let mut sql_example: Option<LitStr> = None;
     let mut standard_args: Vec<(Option<LitStr>, Option<LitStr>)> = vec![];
     let mut udf_args: Vec<(Option<LitStr>, Option<LitStr>)> = vec![];
@@ -112,15 +118,9 @@ pub fn user_doc(args: TokenStream, input: TokenStream) -> TokenStream {
     let parser = syn::meta::parser(|meta| {
         if meta.path.is_ident("doc_section") {
             meta.parse_nested_meta(|meta| {
-                if meta.path.is_ident("include") {
-                    doc_section_include = meta.value()?.parse()?;
-                    return Ok(());
-                } else if meta.path.is_ident("label") {
+                if meta.path.is_ident("label") {
                     doc_section_lbl = meta.value()?.parse()?;
                     return Ok(());
-                } else if meta.path.is_ident("description") {
-                    doc_section_desc = meta.value()?.parse()?;
-                    return Ok(());
                 }
                 Ok(())
             })
@@ -131,7 +131,7 @@ pub fn user_doc(args: TokenStream, input: TokenStream) -> TokenStream {
             syntax_example = Some(meta.value()?.parse()?);
             Ok(())
         } else if meta.path.is_ident("alternative_syntax") {
-            alt_syntax_example = Some(meta.value()?.parse()?);
+            alt_syntax_example.push(Some(meta.value()?.parse()?));
             Ok(())
         } else if meta.path.is_ident("sql_example") {
             sql_example = Some(meta.value()?.parse()?);
@@ -192,13 +192,18 @@ pub fn user_doc(args: TokenStream, input: TokenStream) -> TokenStream {
     let input = parse_macro_input!(input as DeriveInput);
     let name = input.clone().ident;
 
-    let doc_section_include: bool = if let Some(doc_section_include) = doc_section_include
-    {
-        doc_section_include.value().parse().unwrap()
-    } else {
-        true
-    };
-
+    if doc_section_lbl.is_none() {
+        eprintln!("label for doc_section should exist");
+    }
+    let label = doc_section_lbl.as_ref().unwrap().value();
+    // Try to find a predefined const by label first.
+    // If there is no match but label exists, default value will be used for include and description
+    let doc_section_option = doc_sections_const().iter().find(|ds| ds.label == label);
+    let (doc_section_include, doc_section_label, doc_section_desc) =
+        match doc_section_option {
+            Some(section) => (section.include, section.label, section.description),
+            None => (true, label.as_str(), None),
+        };
     let doc_section_description = doc_section_desc
         .map(|desc| quote! { Some(#desc)})
         .unwrap_or(quote! { None });
@@ -242,7 +247,7 @@ pub fn user_doc(args: TokenStream, input: TokenStream) -> TokenStream {
         })
         .collect::<Vec<_>>();
 
-    let alt_syntax_example = alt_syntax_example.map(|syn| {
+    let alt_syntax_example = alt_syntax_example.iter().map(|syn| {
         quote! {
             .with_alternative_syntax(#syn)
         }
@@ -255,10 +260,10 @@ pub fn user_doc(args: TokenStream, input: TokenStream) -> TokenStream {
             fn doc(&self) -> Option<&datafusion_doc::Documentation> {
                 static DOCUMENTATION: std::sync::LazyLock<datafusion_doc::Documentation> =
                     std::sync::LazyLock::new(|| {
-                        datafusion_doc::Documentation::builder(datafusion_doc::DocSection { include: #doc_section_include, label: #doc_section_lbl, description: #doc_section_description },
+                        datafusion_doc::Documentation::builder(datafusion_doc::DocSection { include: #doc_section_include, label: #doc_section_label, description: #doc_section_description },
                     #description.to_string(), #syntax_example.to_string())
                         #sql_example
-                        #alt_syntax_example
+                        #(#alt_syntax_example)*
                         #(#standard_args)*
                         #(#udf_args)*
                         #(#related_udfs)*
