@@ -492,8 +492,11 @@ fn merge_consecutive_projections(proj: Projection) -> Result<Transformed<Project
                 expr,
                 relation,
                 name,
+                metadata,
             }) => rewrite_expr(*expr, &prev_projection).map(|result| {
-                result.update_data(|expr| Expr::Alias(Alias::new(expr, relation, name)))
+                result.update_data(|expr| {
+                    Expr::Alias(Alias::new(expr, relation, name).with_metadata(metadata))
+                })
             }),
             e => rewrite_expr(e, &prev_projection),
         }
