@@ -22,6 +22,8 @@ use regex_syntax::hir::{Capture, Hir, HirKind, Literal, Look};
 /// Maximum number of regex alternations (`foo|bar|...`) that will be expanded into multiple `LIKE` expressions.
 const MAX_REGEX_ALTERNATIONS_EXPANSION: usize = 4;
 
+const ANY_CHAR_REGEX_PATTERN: &str = ".*";
+
 /// Tries to convert a regexp expression to a `LIKE` or `Eq`/`NotEq` expression.
 ///
 /// This function also validates the regex pattern. And will return error if the
@@ -33,6 +35,8 @@ const MAX_REGEX_ALTERNATIONS_EXPANSION: usize = 4;
 /// - full anchored regex patterns (e.g. `^foo$`) to `= 'foo'`
 /// - partial anchored regex patterns (e.g. `^foo`) to `LIKE 'foo%'`
 /// - combinations (alternatives) of the above, will be concatenated with `OR` or `AND`
+/// - `EQ .*` to NotNull
+/// - `NE .*` means IS EMPTY
 ///
 /// Dev note: unit tests of this function are in `expr_simplifier.rs`, case `test_simplify_regex`.
 pub fn simplify_regex_expr(
@@ -43,6 +47,23 @@ pub fn simplify_regex_expr(
     let mode = OperatorMode::new(&op);
 
     if let Expr::Literal(ScalarValue::Utf8(Some(pattern))) = right.as_ref() {
+        // Handle the special case for ".*" pattern
+        if pattern == ANY_CHAR_REGEX_PATTERN {
+            let new_expr = if mode.not {
+                // not empty
+                let empty_lit = Box::new(lit(""));
+                Expr::BinaryExpr(BinaryExpr {
+                    left,
+                    op: Operator::Eq,
+                    right: empty_lit,
+                })
+            } else {
+                // not null
+                left.is_not_null()
+            };
+            return Ok(new_expr);
+        }
+
         match regex_syntax::Parser::new().parse(pattern) {
             Ok(hir) => {
                 let kind = hir.kind();
