@@ -156,7 +156,10 @@ impl CaseExpr {
                 && else_expr.as_ref().unwrap().as_any().is::<Literal>()
             {
                 EvalMethod::ScalarOrScalar
-            } else if when_then_expr.len() == 1 && else_expr.is_some() {
+            } else if when_then_expr.len() == 1
+                && is_cheap_and_infallible(&(when_then_expr[0].1))
+                && else_expr.as_ref().is_some_and(is_cheap_and_infallible)
+            {
                 EvalMethod::ExpressionOrExpression
             } else {
                 EvalMethod::NoExpression
@@ -559,6 +562,29 @@ impl PhysicalExpr for CaseExpr {
             )?))
         }
     }
+
+    fn fmt_sql(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "CASE ")?;
+        if let Some(e) = &self.expr {
+            e.fmt_sql(f)?;
+            write!(f, " ")?;
+        }
+
+        for (w, t) in &self.when_then_expr {
+            write!(f, "WHEN ")?;
+            w.fmt_sql(f)?;
+            write!(f, " THEN ")?;
+            t.fmt_sql(f)?;
+            write!(f, " ")?;
+        }
+
+        if let Some(e) = &self.else_expr {
+            write!(f, "ELSE ")?;
+            e.fmt_sql(f)?;
+            write!(f, " ")?;
+        }
+        write!(f, "END")
+    }
 }
 
 /// Create a CASE expression
@@ -583,6 +609,7 @@ mod tests {
     use datafusion_common::tree_node::{Transformed, TransformedResult, TreeNode};
     use datafusion_expr::type_coercion::binary::comparison_coercion;
     use datafusion_expr::Operator;
+    use datafusion_physical_expr_common::physical_expr::fmt_sql;
 
     #[test]
     fn case_with_expr() -> Result<()> {
@@ -789,9 +816,18 @@ mod tests {
     }
 
     fn case_test_batch1() -> Result<RecordBatch> {
-        let schema = Schema::new(vec![Field::new("a", DataType::Int32, true)]);
+        let schema = Schema::new(vec![
+            Field::new("a", DataType::Int32, true),
+            Field::new("b", DataType::Int32, true),
+            Field::new("c", DataType::Int32, true),
+        ]);
         let a = Int32Array::from(vec![Some(1), Some(0), None, Some(5)]);
-        let batch = RecordBatch::try_new(Arc::new(schema), vec![Arc::new(a)])?;
+        let b = Int32Array::from(vec![Some(3), None, Some(14), Some(7)]);
+        let c = Int32Array::from(vec![Some(0), Some(-3), Some(777), None]);
+        let batch = RecordBatch::try_new(
+            Arc::new(schema),
+            vec![Arc::new(a), Arc::new(b), Arc::new(c)],
+        )?;
         Ok(batch)
     }
 
@@ -1282,18 +1318,8 @@ mod tests {
             lit(2i32),
             &batch.schema(),
         )?;
-        let then = binary(
-            col("a", &schema)?,
-            Operator::Plus,
-            lit(1i32),
-            &batch.schema(),
-        )?;
-        let else_expr = binary(
-            col("a", &schema)?,
-            Operator::Minus,
-            lit(1i32),
-            &batch.schema(),
-        )?;
+        let then = col("b", &schema)?;
+        let else_expr = col("c", &schema)?;
         let expr = CaseExpr::try_new(None, vec![(when, then)], Some(else_expr))?;
         assert!(matches!(
             expr.eval_method,
@@ -1305,7 +1331,7 @@ mod tests {
             .expect("Failed to convert to array");
         let result = as_int32_array(&result).expect("failed to downcast to Int32Array");
 
-        let expected = &Int32Array::from(vec![Some(2), Some(1), None, Some(4)]);
+        let expected = &Int32Array::from(vec![Some(3), None, Some(777), None]);
 
         assert_eq!(expected, result);
         Ok(())
@@ -1378,4 +1404,35 @@ mod tests {
                 comparison_coercion(&left_type, right_type)
             })
     }
+
+    #[test]
+    fn test_fmt_sql() -> Result<()> {
+        let schema = Schema::new(vec![Field::new("a", DataType::Utf8, true)]);
+
+        // CASE WHEN a = 'foo' THEN 123.3 ELSE 999 END
+        let when = binary(col("a", &schema)?, Operator::Eq, lit("foo"), &schema)?;
+        let then = lit(123.3f64);
+        let else_value = lit(999i32);
+
+        let expr = generate_case_when_with_type_coercion(
+            None,
+            vec![(when, then)],
+            Some(else_value),
+            &schema,
+        )?;
+
+        let display_string = expr.to_string();
+        assert_eq!(
+            display_string,
+            "CASE WHEN a@0 = foo THEN 123.3 ELSE TRY_CAST(999 AS Float64) END"
+        );
+
+        let sql_string = fmt_sql(expr.as_ref()).to_string();
+        assert_eq!(
+            sql_string,
+            "CASE WHEN a = foo THEN 123.3 ELSE TRY_CAST(999 AS Float64) END"
+        );
+
+        Ok(())
+    }
 }
