@@ -123,6 +123,12 @@ impl PhysicalExpr for TryCastExpr {
             self.cast_type.clone(),
         )))
     }
+
+    fn fmt_sql(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "TRY_CAST(")?;
+        self.expr.fmt_sql(f)?;
+        write!(f, " AS {:?})", self.cast_type)
+    }
 }
 
 /// Return a PhysicalExpression representing `expr` casted to
@@ -158,6 +164,7 @@ mod tests {
         },
         datatypes::*,
     };
+    use datafusion_physical_expr_common::physical_expr::fmt_sql;
 
     // runs an end-to-end test of physical type cast
     // 1. construct a record batch with a column "a" of type A
@@ -573,4 +580,26 @@ mod tests {
             .with_precision_and_scale(precision, scale)
             .unwrap()
     }
+
+    #[test]
+    fn test_fmt_sql() -> Result<()> {
+        let schema = Schema::new(vec![Field::new("a", DataType::Int32, true)]);
+
+        // Test numeric casting
+        let expr = try_cast(col("a", &schema)?, &schema, DataType::Int64)?;
+        let display_string = expr.to_string();
+        assert_eq!(display_string, "TRY_CAST(a@0 AS Int64)");
+        let sql_string = fmt_sql(expr.as_ref()).to_string();
+        assert_eq!(sql_string, "TRY_CAST(a AS Int64)");
+
+        // Test string casting
+        let schema = Schema::new(vec![Field::new("b", DataType::Utf8, true)]);
+        let expr = try_cast(col("b", &schema)?, &schema, DataType::Int32)?;
+        let display_string = expr.to_string();
+        assert_eq!(display_string, "TRY_CAST(b@0 AS Int32)");
+        let sql_string = fmt_sql(expr.as_ref()).to_string();
+        assert_eq!(sql_string, "TRY_CAST(b AS Int32)");
+
+        Ok(())
+    }
 }
