@@ -92,6 +92,12 @@ impl DisplayAs for CoalescePartitionsExec {
                 }
                 None => write!(f, "CoalescePartitionsExec"),
             },
+            DisplayFormatType::TreeRender => match self.fetch {
+                Some(fetch) => {
+                    write!(f, "limit: {fetch}")
+                }
+                None => write!(f, ""),
+            },
         }
     }
 }
