@@ -54,6 +54,7 @@ use datafusion_physical_expr::{
     ExprBoundaries, PhysicalExpr,
 };
 
+use datafusion_physical_expr_common::physical_expr::fmt_sql;
 use futures::stream::{Stream, StreamExt};
 use log::trace;
 
@@ -329,6 +330,9 @@ impl DisplayAs for FilterExec {
                 };
                 write!(f, "FilterExec: {}{}", self.predicate, display_projections)
             }
+            DisplayFormatType::TreeRender => {
+                write!(f, "predicate={}", fmt_sql(self.predicate.as_ref()))
+            }
         }
     }
 }
