@@ -108,6 +108,12 @@ impl DisplayAs for GlobalLimitExec {
                     self.fetch.map_or("None".to_string(), |x| x.to_string())
                 )
             }
+            DisplayFormatType::TreeRender => {
+                if let Some(fetch) = self.fetch {
+                    writeln!(f, "limit={}", fetch)?;
+                }
+                write!(f, "skip={}", self.skip)
+            }
         }
     }
 }
@@ -261,6 +267,9 @@ impl DisplayAs for LocalLimitExec {
             DisplayFormatType::Default | DisplayFormatType::Verbose => {
                 write!(f, "LocalLimitExec: fetch={}", self.fetch)
             }
+            DisplayFormatType::TreeRender => {
+                write!(f, "limit={}", self.fetch)
+            }
         }
     }
 }
