@@ -223,6 +223,10 @@ impl DisplayAs for RecursiveQueryExec {
                     self.name, self.is_distinct
                 )
             }
+            DisplayFormatType::TreeRender => {
+                // TODO: collect info
+                write!(f, "")
+            }
         }
     }
 }
