@@ -22,9 +22,9 @@ use datafusion_common::{
     exec_datafusion_err, internal_err, plan_datafusion_err, RecursionUnnestOption,
     Result, ScalarValue, TableReference, UnnestOptions,
 };
+use datafusion_expr::dml::InsertOp;
 use datafusion_expr::expr::{Alias, Placeholder, Sort};
 use datafusion_expr::expr::{Unnest, WildcardOptions};
-use datafusion_expr::ExprFunctionExt;
 use datafusion_expr::{
     expr::{self, InList, WindowFunction},
     logical_plan::{PlanType, StringifiedPlan},
@@ -33,6 +33,7 @@ use datafusion_expr::{
     JoinConstraint, JoinType, Like, Operator, TryCast, WindowFrame, WindowFrameBound,
     WindowFrameUnits,
 };
+use datafusion_expr::{ExprFunctionExt, WriteOp};
 use datafusion_proto_common::{from_proto::FromOptionalField, FromProtoError as Error};
 
 use crate::protobuf::plan_type::PlanTypeEnum::{
@@ -217,6 +218,21 @@ impl From<protobuf::JoinConstraint> for JoinConstraint {
     }
 }
 
+impl From<protobuf::dml_node::Type> for WriteOp {
+    fn from(t: protobuf::dml_node::Type) -> Self {
+        match t {
+            protobuf::dml_node::Type::Update => WriteOp::Update,
+            protobuf::dml_node::Type::Delete => WriteOp::Delete,
+            protobuf::dml_node::Type::InsertAppend => WriteOp::Insert(InsertOp::Append),
+            protobuf::dml_node::Type::InsertOverwrite => {
+                WriteOp::Insert(InsertOp::Overwrite)
+            }
+            protobuf::dml_node::Type::InsertReplace => WriteOp::Insert(InsertOp::Replace),
+            protobuf::dml_node::Type::Ctas => WriteOp::Ctas,
+        }
+    }
+}
+
 pub fn parse_expr(
     proto: &protobuf::LogicalExprNode,
     registry: &dyn FunctionRegistry,
@@ -511,6 +527,7 @@ pub fn parse_expr(
         ))),
         ExprType::Wildcard(protobuf::Wildcard { qualifier }) => {
             let qualifier = qualifier.to_owned().map(|x| x.try_into()).transpose()?;
+            #[expect(deprecated)]
             Ok(Expr::Wildcard {
                 qualifier,
                 options: Box::new(WildcardOptions::default()),
