@@ -67,7 +67,7 @@ use datafusion_common::{
 use datafusion_expr::dml::CopyTo;
 use datafusion_expr::expr::{
     self, Between, BinaryExpr, Case, Cast, GroupingSet, InList, Like, ScalarFunction,
-    Unnest,
+    Unnest, WildcardOptions,
 };
 use datafusion_expr::logical_plan::{Extension, UserDefinedLogicalNodeCore};
 use datafusion_expr::{
@@ -2029,7 +2029,11 @@ fn roundtrip_unnest() {
 
 #[test]
 fn roundtrip_wildcard() {
-    let test_expr = wildcard();
+    #[expect(deprecated)]
+    let test_expr = Expr::Wildcard {
+        qualifier: None,
+        options: Box::new(WildcardOptions::default()),
+    };
 
     let ctx = SessionContext::new();
     roundtrip_expr_test(test_expr, ctx);
@@ -2037,7 +2041,11 @@ fn roundtrip_wildcard() {
 
 #[test]
 fn roundtrip_qualified_wildcard() {
-    let test_expr = qualified_wildcard("foo");
+    #[expect(deprecated)]
+    let test_expr = Expr::Wildcard {
+        qualifier: Some("foo".into()),
+        options: Box::new(WildcardOptions::default()),
+    };
 
     let ctx = SessionContext::new();
     roundtrip_expr_test(test_expr, ctx);
