@@ -16,11 +16,14 @@
 // under the License.
 
 //! [`DFParser`]: DataFusion SQL Parser based on [`sqlparser`]
+//!
+//! This parser implements DataFusion specific statements such as
+//! `CREATE EXTERNAL TABLE`
 
 use std::collections::VecDeque;
 use std::fmt;
 
-use sqlparser::ast::ExprWithAlias;
+use sqlparser::ast::{ExprWithAlias, OrderByOptions};
 use sqlparser::tokenizer::TokenWithSpan;
 use sqlparser::{
     ast::{
@@ -43,12 +46,23 @@ fn parse_file_type(s: &str) -> Result<String, ParserError> {
     Ok(s.to_uppercase())
 }
 
-/// DataFusion specific EXPLAIN (needed so we can EXPLAIN datafusion
-/// specific COPY and other statements)
+/// DataFusion specific `EXPLAIN`
+///
+/// Syntax:
+/// ```sql
+/// EXPLAIN <ANALYZE> <VERBOSE> [FORMAT format] statement
+///```
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct ExplainStatement {
+    /// `EXPLAIN ANALYZE ..`
     pub analyze: bool,
+    /// `EXPLAIN .. VERBOSE ..`
     pub verbose: bool,
+    /// `EXPLAIN .. FORMAT `
+    pub format: Option<String>,
+    /// The statement to analyze. Note this is a DataFusion [`Statement`] (not a
+    /// [`sqlparser::ast::Statement`] so that we can use `EXPLAIN`, `COPY`, and other
+    /// DataFusion specific statements
     pub statement: Box<Statement>,
 }
 
@@ -57,6 +71,7 @@ impl fmt::Display for ExplainStatement {
         let Self {
             analyze,
             verbose,
+            format,
             statement,
         } = self;
 
@@ -67,6 +82,9 @@ impl fmt::Display for ExplainStatement {
         if *verbose {
             write!(f, "VERBOSE ")?;
         }
+        if let Some(format) = format.as_ref() {
+            write!(f, "FORMAT {format} ")?;
+        }
 
         write!(f, "{statement}")
     }
@@ -446,7 +464,6 @@ impl<'a> DFParser<'a> {
                         self.parse_copy()
                     }
                     Keyword::EXPLAIN => {
-                        // (TODO parse all supported statements)
                         self.parser.next_token(); // EXPLAIN
                         self.parse_explain()
                     }
@@ -620,15 +637,35 @@ impl<'a> DFParser<'a> {
     pub fn parse_explain(&mut self) -> Result<Statement, ParserError> {
         let analyze = self.parser.parse_keyword(Keyword::ANALYZE);
         let verbose = self.parser.parse_keyword(Keyword::VERBOSE);
+        let format = self.parse_explain_format()?;
+
         let statement = self.parse_statement()?;
 
         Ok(Statement::Explain(ExplainStatement {
             statement: Box::new(statement),
             analyze,
             verbose,
+            format,
         }))
     }
 
+    pub fn parse_explain_format(&mut self) -> Result<Option<String>, ParserError> {
+        if !self.parser.parse_keyword(Keyword::FORMAT) {
+            return Ok(None);
+        }
+
+        let next_token = self.parser.next_token();
+        let format = match next_token.token {
+            Token::Word(w) => Ok(w.value),
+            Token::SingleQuotedString(w) => Ok(w),
+            Token::DoubleQuotedString(w) => Ok(w),
+            _ => self
+                .parser
+                .expected("an explain format such as TREE", next_token),
+        }?;
+        Ok(Some(format))
+    }
+
     /// Parse a SQL `CREATE` statement handling `CREATE EXTERNAL TABLE`
     pub fn parse_create(&mut self) -> Result<Statement, ParserError> {
         if self.parser.parse_keyword(Keyword::EXTERNAL) {
@@ -708,8 +745,7 @@ impl<'a> DFParser<'a> {
 
         Ok(OrderByExpr {
             expr,
-            asc,
-            nulls_first,
+            options: OrderByOptions { asc, nulls_first },
             with_fill: None,
         })
     }
@@ -756,11 +792,6 @@ impl<'a> DFParser<'a> {
     fn parse_column_def(&mut self) -> Result<ColumnDef, ParserError> {
         let name = self.parser.parse_identifier()?;
         let data_type = self.parser.parse_data_type()?;
-        let collation = if self.parser.parse_keyword(Keyword::COLLATE) {
-            Some(self.parser.parse_object_name(false)?)
-        } else {
-            None
-        };
         let mut options = vec![];
         loop {
             if self.parser.parse_keyword(Keyword::CONSTRAINT) {
@@ -782,7 +813,6 @@ impl<'a> DFParser<'a> {
         Ok(ColumnDef {
             name,
             data_type,
-            collation,
             options,
         })
     }
@@ -1006,7 +1036,6 @@ mod tests {
                 span: Span::empty(),
             },
             data_type,
-            collation: None,
             options: vec![],
         }
     }
@@ -1016,7 +1045,7 @@ mod tests {
         // positive case
         let sql = "CREATE EXTERNAL TABLE t(c1 int) STORED AS CSV LOCATION 'foo.csv'";
         let display = None;
-        let name = ObjectName(vec![Ident::from("t")]);
+        let name = ObjectName::from(vec![Ident::from("t")]);
         let expected = Statement::CreateExternalTable(CreateExternalTable {
             name: name.clone(),
             columns: vec![make_column_def("c1", DataType::Int(display))],
@@ -1314,8 +1343,7 @@ mod tests {
                         quote_style: None,
                         span: Span::empty(),
                     }),
-                    asc,
-                    nulls_first,
+                    options: OrderByOptions { asc, nulls_first },
                     with_fill: None,
                 }]],
                 if_not_exists: false,
@@ -1346,8 +1374,10 @@ mod tests {
                         quote_style: None,
                         span: Span::empty(),
                     }),
-                    asc: Some(true),
-                    nulls_first: None,
+                    options: OrderByOptions {
+                        asc: Some(true),
+                        nulls_first: None,
+                    },
                     with_fill: None,
                 },
                 OrderByExpr {
@@ -1356,8 +1386,10 @@ mod tests {
                         quote_style: None,
                         span: Span::empty(),
                     }),
-                    asc: Some(false),
-                    nulls_first: Some(true),
+                    options: OrderByOptions {
+                        asc: Some(false),
+                        nulls_first: Some(true),
+                    },
                     with_fill: None,
                 },
             ]],
@@ -1395,8 +1427,10 @@ mod tests {
                         span: Span::empty(),
                     })),
                 },
-                asc: Some(true),
-                nulls_first: None,
+                options: OrderByOptions {
+                    asc: Some(true),
+                    nulls_first: None,
+                },
                 with_fill: None,
             }]],
             if_not_exists: false,
@@ -1442,8 +1476,10 @@ mod tests {
                         span: Span::empty(),
                     })),
                 },
-                asc: Some(true),
-                nulls_first: None,
+                options: OrderByOptions {
+                    asc: Some(true),
+                    nulls_first: None,
+                },
                 with_fill: None,
             }]],
             if_not_exists: true,
@@ -1543,6 +1579,7 @@ mod tests {
             let expected = Statement::Explain(ExplainStatement {
                 analyze,
                 verbose,
+                format: None,
                 statement: Box::new(expected_copy),
             });
             assert_eq!(verified_stmt(sql), expected);
@@ -1656,7 +1693,7 @@ mod tests {
     // For error cases, see: `copy.slt`
 
     fn object_name(name: &str) -> CopyToSource {
-        CopyToSource::Relation(ObjectName(vec![Ident::new(name)]))
+        CopyToSource::Relation(ObjectName::from(vec![Ident::new(name)]))
     }
 
     // Based on  sqlparser-rs
