@@ -16,15 +16,16 @@
 // under the License.
 
 use core::fmt;
+use std::ops::ControlFlow;
 
-use sqlparser::ast;
 use sqlparser::ast::helpers::attached_token::AttachedToken;
+use sqlparser::ast::{self, visit_expressions_mut, OrderByKind, SelectFlavor};
 
 #[derive(Clone)]
 pub struct QueryBuilder {
     with: Option<ast::With>,
     body: Option<Box<ast::SetExpr>>,
-    order_by: Vec<ast::OrderByExpr>,
+    order_by_kind: Option<OrderByKind>,
     limit: Option<ast::Expr>,
     limit_by: Vec<ast::Expr>,
     offset: Option<ast::Offset>,
@@ -46,8 +47,8 @@ impl QueryBuilder {
     pub fn take_body(&mut self) -> Option<Box<ast::SetExpr>> {
         self.body.take()
     }
-    pub fn order_by(&mut self, value: Vec<ast::OrderByExpr>) -> &mut Self {
-        self.order_by = value;
+    pub fn order_by(&mut self, value: OrderByKind) -> &mut Self {
+        self.order_by_kind = Some(value);
         self
     }
     pub fn limit(&mut self, value: Option<ast::Expr>) -> &mut Self {
@@ -75,14 +76,13 @@ impl QueryBuilder {
         self
     }
     pub fn build(&self) -> Result<ast::Query, BuilderError> {
-        let order_by = if self.order_by.is_empty() {
-            None
-        } else {
-            Some(ast::OrderBy {
-                exprs: self.order_by.clone(),
+        let order_by = self
+            .order_by_kind
+            .as_ref()
+            .map(|order_by_kind| ast::OrderBy {
+                kind: order_by_kind.clone(),
                 interpolate: None,
-            })
-        };
+            });
 
         Ok(ast::Query {
             with: self.with.clone(),
@@ -105,7 +105,7 @@ impl QueryBuilder {
         Self {
             with: Default::default(),
             body: Default::default(),
-            order_by: Default::default(),
+            order_by_kind: Default::default(),
             limit: Default::default(),
             limit_by: Default::default(),
             offset: Default::default(),
@@ -138,6 +138,7 @@ pub struct SelectBuilder {
     named_window: Vec<ast::NamedWindowDefinition>,
     qualify: Option<ast::Expr>,
     value_table_mode: Option<ast::ValueTableMode>,
+    flavor: Option<SelectFlavor>,
 }
 
 #[allow(dead_code)]
@@ -154,6 +155,11 @@ impl SelectBuilder {
         self.projection = value;
         self
     }
+    pub fn pop_projections(&mut self) -> Vec<ast::SelectItem> {
+        let ret = self.projection.clone();
+        self.projection.clear();
+        ret
+    }
     pub fn already_projected(&self) -> bool {
         !self.projection.is_empty()
     }
@@ -176,6 +182,37 @@ impl SelectBuilder {
         self.lateral_views = value;
         self
     }
+
+    /// Replaces the selection with a new value.
+    ///
+    /// This function is used to replace a specific expression within the selection.
+    /// Unlike the `selection` method which combines existing and new selections with AND,
+    /// this method searches for and replaces occurrences of a specific expression.
+    ///
+    /// This method is primarily used to modify LEFT MARK JOIN expressions.
+    /// When processing a LEFT MARK JOIN, we need to replace the placeholder expression
+    /// with the actual join condition in the selection clause.
+    ///
+    /// # Arguments
+    ///
+    /// * `existing_expr` - The expression to replace
+    /// * `value` - The new expression to set as the selection
+    pub fn replace_mark(
+        &mut self,
+        existing_expr: &ast::Expr,
+        value: &ast::Expr,
+    ) -> &mut Self {
+        if let Some(selection) = &mut self.selection {
+            visit_expressions_mut(selection, |expr| {
+                if expr == existing_expr {
+                    *expr = value.clone();
+                }
+                ControlFlow::<()>::Continue(())
+            });
+        }
+        self
+    }
+
     pub fn selection(&mut self, value: Option<ast::Expr>) -> &mut Self {
         // With filter pushdown optimization, the LogicalPlan can have filters defined as part of `TableScan` and `Filter` nodes.
         // To avoid overwriting one of the filters, we combine the existing filter with the additional filter.
@@ -264,6 +301,10 @@ impl SelectBuilder {
             window_before_qualify: false,
             prewhere: None,
             select_token: AttachedToken::empty(),
+            flavor: match self.flavor {
+                Some(ref value) => value.clone(),
+                None => return Err(Into::into(UninitializedFieldError::from("flavor"))),
+            },
         })
     }
     fn create_empty() -> Self {
@@ -283,6 +324,7 @@ impl SelectBuilder {
             named_window: Default::default(),
             qualify: Default::default(),
             value_table_mode: Default::default(),
+            flavor: Some(SelectFlavor::Standard),
         }
     }
 }
@@ -422,6 +464,7 @@ pub struct TableRelationBuilder {
     with_hints: Vec<ast::Expr>,
     version: Option<ast::TableVersion>,
     partitions: Vec<ast::Ident>,
+    index_hints: Vec<ast::TableIndexHints>,
 }
 
 #[allow(dead_code)]
@@ -450,6 +493,10 @@ impl TableRelationBuilder {
         self.partitions = value;
         self
     }
+    pub fn index_hints(&mut self, value: Vec<ast::TableIndexHints>) -> &mut Self {
+        self.index_hints = value;
+        self
+    }
     pub fn build(&self) -> Result<ast::TableFactor, BuilderError> {
         Ok(ast::TableFactor::Table {
             name: match self.name {
@@ -467,6 +514,7 @@ impl TableRelationBuilder {
             with_ordinality: false,
             json_path: None,
             sample: None,
+            index_hints: self.index_hints.clone(),
         })
     }
     fn create_empty() -> Self {
@@ -477,6 +525,7 @@ impl TableRelationBuilder {
             with_hints: Default::default(),
             version: Default::default(),
             partitions: Default::default(),
+            index_hints: Default::default(),
         }
     }
 }
