@@ -15,4 +15,6 @@
 // specific language governing permissions and limitations
 // under the License.
 
+mod collection;
+mod diagnostic;
 mod plan_to_sql;
